import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**This applet simulates the motion of sun and 12 zodiac signs w.r.t observers at different latitudes in the course of a year.*/
public class EclipticAtDifferentLatitudes extends Applet
{  
   public SunEarthMovie movieFrame;
   public static MonthPanel monthPanel;
   public DayPanel dayPanel;  
   public SunEarthControlPanel staticEarthControlPanel;  
   public MapDetectorPanel mapDetectorPanel;
   public PicturePanel title, signPanel, left, right, bottom;

   /**Starts this applet by initializing all panels and processes needed in this applet.*/
   public void init()
   {
       setBackground(Color.black);
       setLayout(null);
          
       movieFrame = new SunEarthMovie(320,320);
       monthPanel = new MonthPanel(56,378);
       dayPanel = new DayPanel();
       staticEarthControlPanel = new SunEarthControlPanel(525,50,115,245);
       title= new PicturePanel("Title.jpg");
       signPanel= new PicturePanel("signPanel.jpg");
       right= new PicturePanel("right.jpg");
       left= new PicturePanel("left.jpg");
       bottom= new PicturePanel("bottom.jpg");
       mapDetectorPanel = new MapDetectorPanel("world1.jpg"); 

       add(monthPanel);
       add(movieFrame); 
       add(dayPanel);
       add(staticEarthControlPanel);
       add(title);
       add(signPanel);
       add(right);
       add(left);
       add(bottom);
       add(mapDetectorPanel);   
                   
       setVisible(true); 

       setVisible(true);
       monthPanel.setBounds(0,60,56,378);
       staticEarthControlPanel.setBounds(0,434,411,116);
       movieFrame.setBounds(80,70,320,320);  
       dayPanel.setBounds(188,408,200,22);
       signPanel.setBounds(411,60,339,293);
       title.setBounds(0,0,750,60);
       left.setBounds(411,353,8,152);
       right.setBounds(720,353,30,152);
       bottom.setBounds(411,501,339,49);

       mapDetectorPanel.setBounds(419,352,301,152);    
       movieFrame.start();   
       dayPanel.start();
   }
}

/**This window displays the motion of the sun and 12 zodiac signs w.r.t an observer at a particular latitude in the course of a year.*/
class SunEarthMovie extends Canvas implements Runnable
{  
   private static int noPix;
   private static int noOfPixInEcliptic;
   private Pixel horizonDirection;
   public EclipticGenerator ecliptic = new EclipticGenerator(81,1300, 							0.34906585,1.570796327);
   public ZodiacMarker zodiacMarker = new ZodiacMarker(1300, 0.34906585,1.570796327);
   public static Horoscope horo = new Horoscope(1300,16,0.34906585,1.570796327);
   public static double sunAngularSpeed, earthAngularSpeed;
   private final static int obsDistanceToEarth=1300; 
   private static int sunR=0; 
   private static int sunY_Observed, sunZ_Observed, sunX=-1, earthX=0;
   private static int orbitRadius, orbitHeight, horizonRadius, horizonHeight;
   private static Pixel lum=new Pixel(-100,0,0);
   private static final int celestialRadius=180; 
   private static final double screenDistance = 1000;
   private static final int staticSunRadius=24;
   private static int canvasCenterX,canvasCenterY;
   private static int furthestHoro;
   public static boolean freezingDetector=false;
   private Image mImage;
   public static double oldMonth, oneMonth;
  
   public static double absoluteTime;
   public static double latitude, longitude;
   public static double obsAngleToEarth=0.0d;//In radian
   public static double timeStep=2.0d;
   Color sourceColor= new Color(255,255,200);     
   public static boolean newLatitudeDetector=false;
    
   public static Thread SunMotion = null;
   
   /**Constructor.*/
   public SunEarthMovie(int sizeX, int sizeY)
   {
       canvasCenterX=sizeX/2;
       canvasCenterY=sizeY/2;
       setBackground(Color.black); 
       setSize(sizeX, sizeY);                       
   }
 
   /**Starts the simulation.*/
   public void start() {
                      SunMotion = new Thread(this, "SunEarthMovie");
                      SunMotion.start();
                       }

   /**Calculates the positions of the sun and 12 zodiac signs w.r.t an observer at a particular latitude at a particular time in a year.*/
   public void run()
   {  
       double x, y, z, startingTime,thirteenMonth,angle;
       obsAngleToEarth=0.34906585;
       horizonDirection = new Pixel((int)(100*Math.sin(obsAngleToEarth)),0,(int)(100*Math.cos(obsAngleToEarth)));

       startingTime=-3.230136986*StandardConstLib.getQuarterYear()
                    +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3; 

       latitude = StandardConstLib.latitude*0.017453292;
       sunAngularSpeed= StandardConstLib.sunAngularSpeed;
       earthAngularSpeed=StandardConstLib.earthAngularSpeed;

       oneMonth = StandardConstLib.getQuarterYear()/3;     

       oldMonth = startingTime;
       thirteenMonth = 1.103196347*StandardConstLib.getQuarterYear();

       ecliptic.setLatitude(1.570796327 -latitude);
       horo.setLatitude(1.570796327 -latitude);	
       zodiacMarker.setLatitude(1.570796327 -latitude);

       ecliptic.rotateEcliptic(0.0d);
       zodiacMarker.rotateZodiacMarker(0.0d);
       horo.rotateHoro(0.0d);
       furthestHoro = horo.getFurthestHoro();

       noOfPixInEcliptic = ecliptic.noOfPixel;

       TransformationMachine Tran = new TransformationMachine(obsAngleToEarth,obsDistanceToEarth, 
                                                              sunAngularSpeed, earthAngularSpeed,1.570796327 -latitude);

       orbitRadius=(int)Tran.getOrbitRadius();
       horizonRadius= orbitRadius-16;
       orbitHeight=(int)(orbitRadius*Math.sin(0.34906585)); //0.34906585=viewAngle

       horizonHeight = (int)(horizonRadius*Math.sin(0.34906585));
          
       for(absoluteTime=startingTime;absoluteTime<=150000;absoluteTime+=timeStep)
       {   
           if((absoluteTime-oldMonth)>oneMonth)
           {
               if(absoluteTime>thirteenMonth)
               {
                   EclipticAtDifferentLatitudes.monthPanel.setValue(0);
                   absoluteTime-=4*StandardConstLib.getQuarterYear();
               }
               else
               {
                   EclipticAtDifferentLatitudes.monthPanel.setValue(EclipticAtDifferentLatitudes.monthPanel.buttonChoice+1);
               }
               oldMonth=-3*StandardConstLib.getQuarterYear()
                        +(EclipticAtDifferentLatitudes.monthPanel.buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                        -0.230136986*StandardConstLib.getQuarterYear();           }
       
           Tran.transform3D(absoluteTime);
            	   
           x = Tran.getSunX();             	   
           y = Tran.getSunY();
           z = Tran.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
           sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

           lum=new Pixel((int)x,(int)y,(int)z);

           sunX=(int)x;
           sunY_Observed = canvasCenterX -sunR/2 + (int)y;
           sunZ_Observed = canvasCenterY -sunR/2 - (int)z;

           angle = absoluteTime*earthAngularSpeed;

           ecliptic.rotateEcliptic(angle);
           zodiacMarker.rotateZodiacMarker(angle);
           horo.rotateHoro(angle);
           furthestHoro = horo.getFurthestHoro();

           render(); 

           if (newLatitudeDetector)
           {
               Tran = new TransformationMachine (obsAngleToEarth,obsDistanceToEarth, sunAngularSpeed, 
                                                 earthAngularSpeed,1.570796327 -latitude);
      			//phase = longitude*oneDay;

               ecliptic.setLatitude(1.570796327 -latitude);
               horo.setLatitude(1.570796327 -latitude);	
               zodiacMarker.setLatitude(1.570796327 -latitude);
               newLatitudeDetector=false;
           }

           while(freezingDetector)
           {
               Tran.transform3D(absoluteTime);

               x = Tran.getSunX();             	   
               y = Tran.getSunY();
               z = Tran.getSunZ();

               x= celestialRadius*x;
               y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
               z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
               sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 
                 
               lum=new Pixel((int)x,(int)y,(int)z);

               sunX=(int)x;
               sunY_Observed = canvasCenterX -sunR/2 + (int)y;
               sunZ_Observed = canvasCenterY -sunR/2 - (int)z;

               angle = absoluteTime*earthAngularSpeed;

               ecliptic.rotateEcliptic(angle);
               zodiacMarker.rotateZodiacMarker(angle);
               horo.rotateHoro(angle);
               furthestHoro = horo.getFurthestHoro();

               render();  	

               if (newLatitudeDetector)
               {
                   Tran = new TransformationMachine (obsAngleToEarth,obsDistanceToEarth, sunAngularSpeed,
                                                     earthAngularSpeed,1.570796327 -latitude);//phase = longitude*oneDay;

                   ecliptic.setLatitude(1.570796327 -latitude);
                   horo.setLatitude(1.570796327 -latitude);	
                   zodiacMarker.setLatitude(1.570796327 -latitude);
                   newLatitudeDetector=false;
               }
           }
       }                
   }
   
   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           Dimension d= getSize();
           if (checkImage(d))
           {
               Graphics imageGraphics = mImage.getGraphics();
               imageGraphics.setColor(getBackground());
               imageGraphics.fillRect(0,0,d.width,d.height);
               paint(imageGraphics);
               g.drawImage(mImage,0,0,null);
               imageGraphics.dispose();
           }
           g.dispose(); 
       }
   }   

   /**Paints the sun-earth and zodiac signs motion to an observer at a particular latitude.*/
   public void paint(Graphics g)
   {
       int i, j,k, zodiacSize=2, compBrightness;
       float brightness, eBrightness,horizonBrightness;
       Color color;           
       Color c;

       horizonBrightness = Pixel.unitDotProduct(lum,horizonDirection);  
       if(horizonBrightness<0.01) { horizonBrightness =0.01f;} 
       compBrightness = (int)((horizonBrightness+0.3f)*155);   

       if(sunX <0)
       {
           for(i=((noOfPixInEcliptic-1)/2);i<noOfPixInEcliptic-1;i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));

               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()<0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }                   

           for(k=furthestHoro; k < (furthestHoro + 4);k++)
           {
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),                                       		      canvasCenterY- horo.horoPosition[j].getZ()-
                              horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           } 

           for(k=(furthestHoro-1); k >= (furthestHoro - 3);k--)
           {
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),                                       		      canvasCenterY- horo.horoPosition[j].getZ()-
                              horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           }
          
           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed, sunZ_Observed, sunR, sunR);

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()>0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }

           g.setColor(new Color(255,255,255));
           g.drawLine(canvasCenterX, canvasCenterY,canvasCenterX, canvasCenterY+orbitRadius);

           g.setColor(new Color(130,130,160));
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius, 2*orbitRadius,2*orbitRadius);
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitHeight, 2*orbitRadius,2*orbitHeight);

           g.setColor(new Color(compBrightness,compBrightness,compBrightness));
           g.fillOval(canvasCenterX-horizonRadius,canvasCenterY-horizonHeight,2*horizonRadius, 2*horizonHeight);

           g.setColor(new Color(255,255,255));
           g.drawLine(canvasCenterX, canvasCenterY-orbitRadius,canvasCenterX, canvasCenterY);

           g.setColor(Color.green); g.drawString("E", canvasCenterX-4, canvasCenterY-horizonHeight-4);
           g.drawString("W", canvasCenterX-5, canvasCenterY+horizonHeight+13);
           g.drawString("N", canvasCenterX-horizonRadius-12, canvasCenterY+4);
           g.drawString("S", canvasCenterX+horizonRadius+4, canvasCenterY+4);

           for(i=0;i<((noOfPixInEcliptic-1)/2);i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(k=(furthestHoro-8); k < (furthestHoro - 3);k++)
           {	
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),                                     	            canvasCenterY- horo.horoPosition[j].getZ()-
                              horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           }
       }
       else
       {
           for(i=((noOfPixInEcliptic-1)/2);i<noOfPixInEcliptic-1;i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()<0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }

           for(k=furthestHoro; k < (furthestHoro + 4);k++)
           {
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),                                       		      canvasCenterY- horo.horoPosition[j].getZ()-
                              horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           } 

           for(k=(furthestHoro-1); k >= (furthestHoro - 3);k--)
           {
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           }

           g.setColor(new Color(255,255,255));
           g.drawLine(canvasCenterX, canvasCenterY,canvasCenterX, canvasCenterY+orbitRadius);

           g.setColor(new Color(130,130,160));
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius,2*orbitRadius,2*orbitRadius);
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitHeight,2*orbitRadius,2*orbitHeight);

           g.setColor(new Color(compBrightness,compBrightness,compBrightness));
           g.fillOval(canvasCenterX-horizonRadius,canvasCenterY-horizonHeight,2*horizonRadius, 2*horizonHeight);

           g.setColor(new Color(255,255,255));
           g.drawLine(canvasCenterX, canvasCenterY-orbitRadius,canvasCenterX, canvasCenterY);

           g.setColor(Color.green);
           g.drawString("E", canvasCenterX-4, canvasCenterY-horizonHeight-4);
           g.drawString("W", canvasCenterX-5, canvasCenterY+horizonHeight+13);
           g.drawString("N", canvasCenterX-horizonRadius-12,canvasCenterY+4);
           g.drawString("S", canvasCenterX+horizonRadius+4, canvasCenterY+4);

           for(i=0;i<((noOfPixInEcliptic-1)/2);i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()>0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                   canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }            

           for(k=(furthestHoro-8); k < (furthestHoro - 3);k++)
           {	
               j=k;	
               if(j>11){j-=12;}
               if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           }

           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed,sunZ_Observed, sunR, sunR);
       }
   }

   protected boolean checkImage(Dimension d)
   {
       if (d.width==0 || d.height == 0) return false;
       if (mImage == null || mImage.getWidth(null) != d.width || mImage.getHeight(null) 		!=d.height)
       {
           mImage=createImage(d.width,d.height);   
       }
       return true;
   }
}

/**This panel tells users the day of a year that the motion on the screen correspond to.*/
class DayPanel extends Panel implements Runnable
{
   private Thread dayTimer;       
   private int dayOfTheYear, newDay=61;
   private static String[] numberString={"0","1","2","3","4","5","6","7","8","9"};

   //comments:absoluteTime=0 at September Equinox =264.33days in one year.
    
   /**Constructor.*/
   public DayPanel()
   {
       setSize(100,20);
   }

   /**Starts the panel.*/
   public void start()
   {
       dayTimer = new Thread(this, "DayPanel");
       dayTimer.start();
   }

   /**Calculates the time and displays it if it is a new day.*/
   public void run()
   {
       dayOfTheYear= ((StandardConstLib.startingMonth-1)*365)/12;
       newDay=dayOfTheYear;
   
       while(dayTimer.isAlive())
       { 
           newDay = (int)(SunEarthMovie.absoluteTime*StandardConstLib.sunAngularSpeed*58.09155423+264.33334d+1);

           if(newDay!=dayOfTheYear)
           {
               repaint();
               dayOfTheYear=newDay;
           }
           try{
                  Thread.sleep(300);
              }catch(InterruptedException e){} 
       }
   }

   /**Paints the day of the year on the screen.*/
   public void paint(Graphics g)
   {
       int newDayCounter;
       double angle;
       float floatAngle;
       String finalString;

       g.setColor(Color.cyan);

       newDayCounter=newDay;

       finalString=Integer.toString(newDayCounter);
       finalString=finalString.concat(" of 365 days");
       g.drawString(finalString, 1,10);        
   }
}

/**This class contains user interface that allows users to freeze, unfreeze, slow down and speed up the applet.*/
class SunEarthControlPanel extends Canvas implements MouseListener
{
    private BufferedImage background;
    private BufferedImage[] speedImage = new BufferedImage[4];
    private BufferedImage[] timeImage = new BufferedImage[4];
    private Image mImage;
    private String[] speedPicName = {"speedL.jpg","speedN.jpg","speedMinusB.jpg","speedPlusB.jpg"};
    private String[] timePicName = {"freezetime1.jpg","freezetime2.jpg","unfreezetime1.jpg","unfreezetime2.jpg"};
    private int timeX = 0, timeY = 12;
    private int speedX = 145, speedY = 12;   
    private int speedW = 112, speedH =37;
    private int timeW = 121, timeH =37;
    private int height, width;

    public static int speedNo = 1, timeNo = 0, resetNo = 0;

    /**Constructor.*/
    public SunEarthControlPanel(int sizeX, int sizeY, int timePos, int speedPos)
    {
        timeX=timePos;
        speedX=speedPos;

        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(speedPicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   speedImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }
   
        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(timePicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   timeImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }

        width=sizeX;
        height=sizeY;
        setBackground(new Color(6,76,73));
        addMouseListener(this);
        setVisible(true);
    }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImage=createImage(width, height);   
           Graphics imageGraphics = mImage.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImage,0,0,this);
           imageGraphics.dispose();
           g.dispose();
       }
   }
     
    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        g.setColor(new Color(75,75,75));
        g.fillRect(0,0,411,5);
        g.setColor(new Color(0,109,104));
        g.fillRect(0,5,411,50);
        g.setColor(new Color(4,76,73));
        g.fillRect(0,55,411,64);
        g.drawImage(speedImage[speedNo], speedX, speedY, this);
        g.drawImage(timeImage[timeNo], timeX, timeY, this);
    }

    /**Detects and records the button that is pressed.*/
    public void mousePressed(MouseEvent e)
    {
       int mouseX, mouseY;
       mouseX = e.getX();
       mouseY = e.getY();

       if((mouseX>timeX)&&(mouseX<timeX+timeW)&&(mouseY>timeY)&&(mouseY<timeY+timeH))
       {
           timeNo+=1;
       }
       
       if(timeNo==0)
       {
           if((mouseX>speedX+3)&&(mouseX<speedX+56)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=3;
           }
       
           if((mouseX>speedX+61)&&(mouseX<speedX+111)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=2;
           }
       }
       render();
    }

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
 
    /**Resets the state of the applet when when a button that was previous pressed on is now released.*/
    public void mouseReleased(MouseEvent e)
    {
        if(timeNo==1)
        {
            timeNo=2; speedNo=0;
            SunEarthMovie.freezingDetector=true;
        }
        else if(timeNo==3)
        {
            timeNo=0; speedNo=1;
            SunEarthMovie.freezingDetector=false;
        }

        if(speedNo==2)
        {
            speedNo=1;
            if(SunEarthMovie.timeStep>0.51)
            {SunEarthMovie.timeStep-=0.251;}
            else
            { SunEarthMovie.timeStep=SunEarthMovie.timeStep/2;}
        }

        if(speedNo==3)
        {   
            speedNo=1;         
            if(SunEarthMovie.timeStep<15&&SunEarthMovie.timeStep>=0.251)
            { 
                SunEarthMovie.timeStep+=0.251;
            }
            else if(SunEarthMovie.timeStep<0.251)
            {SunEarthMovie.timeStep*=2;}
        }

        render();
    }
}

/**This panel displays a world map and allows users to change the position of the observer by clicking on the world map.*/
class MapDetectorPanel extends Panel implements MouseListener
{
   BufferedImage mImage;
   Image mImageBackground;
   int imageWidth, imageHeight;
   int pointX, pointY;

   double longitudeRatio;    

   /**Constructor that creates a map in memory by opening a picture file.*/
   public MapDetectorPanel(String filename)
   { 
      try{
            InputStream in =getClass().getResourceAsStream(filename);
            JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
            mImage= decoder.decodeAsBufferedImage();
            in.close();
         }catch(Exception e){}  

       setSize(mImage.getWidth(), mImage.getHeight()); 
       addMouseListener(this); 
       imageWidth = mImage.getWidth();
       imageHeight = mImage.getHeight();
       pointX = (int)(imageWidth*(StandardConstLib.longitude+0.2)); 
       pointY = (int)(imageHeight*(0.5-(double)StandardConstLib.latitude/180)); 
   }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImageBackground=createImage(imageWidth, imageHeight);   
           Graphics imageGraphics = mImageBackground.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImageBackground,0,0,null);
           imageGraphics.dispose();
           g.dispose();
       }
   }

    /**Paints the map.*/
   public void paint(Graphics g)
   {
       g.drawImage(mImage, 0, 0, this);
       g.setColor(new Color(255,220,180));
       g.fillOval(pointX-5,pointY-3, 10, 5);
   }
   
   /**Sets the new position of the observer if a user selects a new position on the map*/
   public void mousePressed(MouseEvent e) 
   {
       SunEarthMovie.latitude = 3.141592654*(0.5-e.getY()/(double)imageHeight);
       longitudeRatio= e.getX()/(double)imageWidth;

       if(longitudeRatio<=0.2)           
       {
           SunEarthMovie.longitude = longitudeRatio+0.8d;
       }          
       else
       {
           SunEarthMovie.longitude = longitudeRatio-0.2d;
       } 	

       SunEarthMovie.newLatitudeDetector=true;
       pointX = e.getX(); 
       pointY = e.getY();          
       render();
   }

    public void mouseClicked(MouseEvent e) { }

    public void mouseEntered(MouseEvent e) { }

    public void mouseExited(MouseEvent e) { }

    public void mouseReleased(MouseEvent e) { }
}

/**This panel displays the user interface that allows users to select any particular month in a year.*/ 
class MonthPanel extends Panel implements MouseListener, MouseMotionListener
{
    private BufferedImage background;
    private BufferedImage[] button = new BufferedImage[12];
    private Image mImageBackground;
    public int mouseX1, mouseY1;
    private int buttonX=0, buttonY=332, buttonWidth=50, buttonHeight=20, buttonYDis=24;
    public int buttonChoice = 0;
    private String[] file = {"Mth01.jpg","Mth02.jpg","Mth03.jpg","Mth04.jpg","Mth05.jpg","Mth06.jpg","Mth07.jpg","Mth08.jpg","Mth09.jpg","Mth10.jpg","Mth11.jpg","Mth12.jpg"};
    private int panelHeight, panelWidth;
    private double quarterYear;

    /**Constructor.*/
    public MonthPanel(int sizeX, int sizeY)
    {
        int i;

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        try{
               InputStream in =getClass().getResourceAsStream("MonthButton.jpg");
               JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
               background = decoder.decodeAsBufferedImage();
               in.close(); 
           }catch(Exception e){} 

        panelWidth = sizeX;
        panelHeight = sizeY;
        addMouseListener(this);
        addMouseMotionListener(this);

        setValue(StandardConstLib.startingMonth-1);

        quarterYear=StandardConstLib.getQuarterYear();
           
        SunEarthMovie.absoluteTime =-3*quarterYear+quarterYear*StandardConstLib.startingMonth/3.00d
                                          -0.230136986*quarterYear;//0.230136986 = 21days
    }

    /**Sets the month of the year and repaints the user interface.*/
    public void setValue(int value)
    {
        buttonChoice = value;
        render();
    }

    /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
    protected void render()
    {
        Graphics g=getGraphics();
        if(g!=null)
        {
            mImageBackground=createImage(panelWidth, panelHeight);
            Graphics imageGraphics = mImageBackground.getGraphics();
            paint(imageGraphics);
            g.drawImage(mImageBackground,0,0,null);
            imageGraphics.dispose();
            g.dispose();
        }
    }     

    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        int i;
        g.drawImage(background,0,0,this);
        g.drawImage(button[buttonChoice],buttonX, buttonY-(buttonChoice+1)*buttonYDis,this);
    }

    /**Resets the month of the year if a particular button corresponding to that month is pressed.*/
    public void mousePressed(MouseEvent e)
    {
        double temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if(mouseX1>buttonX&&mouseX1<buttonX+buttonWidth)
        {
            temp = (double)(buttonY-mouseY1)/buttonYDis;
            if((temp-(int)temp<(double)buttonHeight/buttonYDis)&&buttonChoice!=(int)temp)
            {
                buttonChoice = (int)temp;

                SunEarthMovie.absoluteTime=-3*quarterYear+quarterYear*(double)(buttonChoice+1)/3.00d
                                           -0.230136986*quarterYear;
                           
                SunEarthMovie.oldMonth=-3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                                       -0.230136986*StandardConstLib.getQuarterYear();
                render();
            }
        }
    }

    public void mouseReleased(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}

    public void mouseDragged(MouseEvent e){}

    public void mouseMoved(MouseEvent e){}
}


/**This class stores important constants that are used in this applet.*/
class StandardConstLib
{    
   public final static int startingMonth=3;
   public final static double obsAngleToEarth=0.00d;
   public final static double latitude=0.00d;
   public final static double longitude = 0.5;
   public final static double sunAngularSpeed=0.00008291, earthAngularSpeed=0.03034506;
   
   public static double getQuarterYear()
   {
       return 1.570796327/sunAngularSpeed;
   }
}



