import java.awt.*;

/**This class creates 12 spherical zodiac signs on the ecliptic by calling MappedReflectiveSphere.*/
public class Horoscope
{
   private static final double earthAXISCOS=0.917060074;//cos(23.5)
   private static final double earthAXISSIN=0.398749068; //sin(23.5)
   public Pixel[] horoPosition = new Pixel[12];
   public Pixel[] oriHoroPosition = new Pixel[12];
   private static int obsDistanceToEarth;
   private static int celestialRadius=180;
   private static int screenDistance=1000;
   public int noOfHoroscope=12;
   public int noPixHoroSphere;
   private Pixel lum = new Pixel(100,10, -20);
   private Color sourceColor = new Color(230,230,255);
   private int furthestHoro;
   private double furthestDistance;
   private double latRotCos, latRotSin, viewAngleCos, viewAngleSin;

    public MappedReflectiveSphere[] horoSphere = new MappedReflectiveSphere[12];

    public String[] horoscope= {"capricornus.jpg", "aquarius.jpg", "pisces.jpg",
                                "aries.jpg", "taurus.jpg", "gemini.jpg", 
                                "cancer.jpg", "leo.jpg", "virgo.jpg","libra.jpg",
                                "scorpius.jpg", "sagittarius.jpg"};
    /*Constructor.*/
   public Horoscope(int obs_Distance, int sphereRadius, double viewAngle, double latRot)
    {  
       int i;
       double angleStep, x, y, z;
       float brightness;

       viewAngleCos = Math.cos(viewAngle);
       viewAngleSin = Math.sin(viewAngle);
       latRotCos = Math.cos(latRot);
       latRotSin = Math.sin(latRot);

       angleStep = 6.283185308/12; 
       obsDistanceToEarth=obs_Distance;

       for(i=0;i<noOfHoroscope;i++)
       {
           x = -celestialRadius*Math.sin(angleStep*i+3.403392041);
           y = celestialRadius*earthAXISCOS*Math.cos(angleStep*i+3.403392041);
           z = celestialRadius*earthAXISSIN*Math.cos(angleStep*i+3.403392041);

           oriHoroPosition[i]=new Pixel((int)x,(int)y,(int)z);

           horoSphere[i] = new MappedReflectiveSphere(sphereRadius, 0.18, 1);
           horoSphere[i].createMap(horoscope[i]);
           horoSphere[i].mappingXZrotation(0.0d,0.5);
           horoSphere[i].lumination(lum , sourceColor);
       }

        noPixHoroSphere = horoSphere[0].getPixelCount();
    }
  
   /**Calculates the motion of the 12 zodiac signs around the earth*/
   public void rotateHoro(double angle)					  
   {  
       double x1, y1, z1, x, y, z, cosRotation, sinRotation;
       furthestHoro = 0;
       furthestDistance = 0.0;

       cosRotation = Math.cos(angle);
       sinRotation = Math.sin(angle);

       for(int i=0;i<noOfHoroscope;i++)
       {
           x1 = cosRotation*oriHoroPosition[i].getX()+sinRotation*oriHoroPosition[i].getY();
           y1 =-sinRotation*oriHoroPosition[i].getX()+cosRotation*oriHoroPosition[i].getY();	
           z1 = oriHoroPosition[i].getZ();

           x = viewAngleCos*x1 + viewAngleSin*latRotSin*y1+viewAngleSin*latRotCos*z1;
           y = latRotCos*y1 - latRotSin*z1;
           z = -viewAngleSin*x1 + viewAngleCos*latRotSin*y1+viewAngleCos*latRotCos*z1;

           y = y*screenDistance/(obsDistanceToEarth-x);
           z = z*screenDistance/(obsDistanceToEarth-x);

           horoPosition[i] = new Pixel((int)x, (int)y, (int)z);  
           if(x<furthestDistance)
           {
               furthestDistance = x;
               furthestHoro = i;
           }
       }                         
   }

   /**Sets the latitude of the observer.*/
   public void setLatitude(double latRot)
   {
       latRotCos = Math.cos(latRot);
       latRotSin = Math.sin(latRot);	
   }

   /**Sets the perspective of the observer.*/
   public void setViewAngle(double viewAngle)
   {
       viewAngleCos = Math.cos(viewAngle);
       viewAngleSin = Math.sin(viewAngle);
   }

   /**Returns the furthest zodiac sign from the observer.*/
   public int getFurthestHoro()
   {
       return furthestHoro;
   }
}