import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**This class defines a way in which a 2D longitude-latitude world map is mapped onto a 3D metal-like sphere.*/
class MappedReflectiveSphere
{
   Pixel[] spPixel = new Pixel[900];
   Color[] spOriginalColor = new Color[900];
   private int pixCount=0;  
   private BufferedImage mImage;
   private double beta, reflectivity;

   /**Constructor that creates an array of Pixel that represents a sphere.*/
   public MappedReflectiveSphere(int Radius, double reflect, double betaInput)
   {  
       int pixX, pixY,pixZ, pixY_Limit, pixSize=1;
       for (pixZ=-Radius; pixZ<Radius; pixZ+=pixSize)
       {
           pixY_Limit = (int)Math.sqrt(Radius*Radius - pixZ*pixZ);    		    	    
           for(pixY=-pixY_Limit; pixY<pixY_Limit; pixY+=pixSize)
           {
               pixX=(int)(Math.sqrt(Radius*Radius- pixZ*pixZ- pixY*pixY));
               spPixel[pixCount] = new Pixel(pixX, pixY, pixZ,pixSize);
               pixCount++;	
           }
       }   
       reflectivity = reflect;
       beta = betaInput;
   }

   /**Creates a map in machine memory by inputing a file name.*/
   public void createMap(String filename)
   {     	
       try{
              InputStream in =getClass().getResourceAsStream(filename);
              JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
              mImage= decoder.decodeAsBufferedImage();
              in.close();
          }catch(Exception e){}  
   }

   /**Maps the map onto the sphere and rotates the sphere about the Y-axis.*/
   public void mappingXZrotation(double mappingAngle, double mapMidPoint) // 0<=midPoint<=1
   {
       double cosMappingAngle=Math.cos(mappingAngle);
       double sinMappingAngle=Math.sin(mappingAngle);
       double sitaZ,phiXY;
       float pixelUnitX, pixelUnitY, pixelUnitZ;
       double mapUnitX, mapUnitY, mapUnitZ, XYradius; 
       int mapWidth = mImage.getWidth();
       int mapHeight = mImage.getHeight();
       int mapX,mapY;
       int colorInteger, red, green, blue;
       Color color;

       for(int i=0;i<pixCount;i++)
       {
           pixelUnitX=spPixel[i].getUnitX();                         
           pixelUnitY=spPixel[i].getUnitY();                         
           pixelUnitZ=spPixel[i].getUnitZ();

           mapUnitX = cosMappingAngle*pixelUnitX - sinMappingAngle* pixelUnitZ;
           mapUnitY = (double)pixelUnitY;
           mapUnitZ = sinMappingAngle*pixelUnitX + cosMappingAngle* pixelUnitZ;

           sitaZ = Math.acos(mapUnitZ);
           XYradius = Math.sqrt(1-mapUnitZ*mapUnitZ);

           if(mapUnitX>=0.0d) {phiXY = Math.asin(mapUnitY/XYradius);}
           else  {phiXY =3.141592654 -Math.asin(mapUnitY/XYradius);}
           mapY = (int)((sitaZ/3.141592654)*(mapHeight-1));                   

           phiXY=phiXY/6.283185307+mapMidPoint;
           if(phiXY>=1){phiXY--;}
           if(phiXY<0){phiXY++;}
           mapX = (int)(phiXY*(mapWidth-1));       

           colorInteger = 16777216 + mImage.getRGB(mapX,mapY);
           red = (int)(colorInteger/65536);
           green = (int)((colorInteger-red*65536)/256);
           blue = colorInteger-65536*red-256*green;      
           color = new Color(red, green, blue);

           spOriginalColor[i] = color;                   
       }
   }

   /**Maps the map onto the sphere and rotates the sphere about the X-axis.*/
   public void mappingYZrotation(double mappingAngle, double mapMidPoint) // 0<=midPoint<=1
   {
       double cosMappingAngle=Math.cos(mappingAngle);
       double sinMappingAngle=Math.sin(mappingAngle);
       double sitaZ,phiXY;
       float pixelUnitX, pixelUnitY, pixelUnitZ;
       double mapUnitX, mapUnitY, mapUnitZ, XYradius; 
       int mapWidth = mImage.getWidth();
       int mapHeight = mImage.getHeight();
       int mapX,mapY;
       int colorInteger, red, green, blue;
       Color color;

       for(int i=0;i<pixCount;i++)
       {
           pixelUnitX=spPixel[i].getUnitX();                         
           pixelUnitY=spPixel[i].getUnitY();                         
           pixelUnitZ=spPixel[i].getUnitZ();

           mapUnitX = (double)pixelUnitX;
           mapUnitY = cosMappingAngle*pixelUnitY+sinMappingAngle*pixelUnitZ;
           mapUnitZ = -sinMappingAngle*pixelUnitY+cosMappingAngle*pixelUnitZ;

           sitaZ = Math.acos(mapUnitZ);  
           mapY = (int)((sitaZ/3.141592654)*(mapHeight-1));                   

           XYradius = Math.sqrt(1-mapUnitZ*mapUnitZ);
           phiXY = Math.asin(mapUnitY/XYradius);
           mapY = (int)((sitaZ/3.141592654)*(mapHeight-1));
           if (mapY==mapHeight){mapY--;} 

           phiXY=phiXY/6.283185307+mapMidPoint;
           if(phiXY>=1){phiXY--;}
           if(phiXY<0){phiXY++;}
           mapX = (int)(phiXY*mapWidth);

           colorInteger = 16777216 + mImage.getRGB(mapX,mapY);
           red = (int)(colorInteger/65536);
           green = (int)((colorInteger-red*65536)/256);
           blue = colorInteger-65536*red-256*green;      
           color = new Color(red, green, blue);
           spOriginalColor[i] = color;                    
       }
   }

   /**Maps the map onto the sphere and rotates the sphere for 23.5 degree about the X-axis and follows by a rotation about Y-axis.*/
   public void mapping3Drotation(double mappingAngle, double mapMidPoint) // 0<=midPoint<=1
   {
       double cosMappingAngle=Math.cos(mappingAngle);
       double sinMappingAngle=Math.sin(mappingAngle);
       double sitaZ,phiXY;
       float pixelUnitX, pixelUnitY, pixelUnitZ;
       double mapUnitX, mapUnitY, mapUnitZ, XYradius; 
       int mapWidth = mImage.getWidth();
       int mapHeight = mImage.getHeight();
       int mapX,mapY;
       int colorInteger, red, green, blue;
       Color color;

       for(int i=0;i<pixCount;i++)
       {
           pixelUnitX=spPixel[i].getUnitX();                         
           pixelUnitY=spPixel[i].getUnitY();                         
           pixelUnitZ=spPixel[i].getUnitZ();
           //0.917060074 =cos23.5                   
           //0.398749068 =sin23.5
           mapUnitX = cosMappingAngle*pixelUnitX - sinMappingAngle* pixelUnitZ;
           mapUnitY = 0.917060074*pixelUnitY-0.398749068*(sinMappingAngle*pixelUnitX + cosMappingAngle* pixelUnitZ);

           mapUnitZ = 0.398749068*pixelUnitY+0.917060074*(sinMappingAngle*pixelUnitX + cosMappingAngle* pixelUnitZ);

           sitaZ = Math.acos(mapUnitZ);
           XYradius = Math.sqrt(1-mapUnitZ*mapUnitZ);
           if(mapUnitX>=0.0d) {phiXY = Math.asin(mapUnitY/XYradius);}
           else  {phiXY =3.141592654 -Math.asin(mapUnitY/XYradius);}
           mapY = (int)((sitaZ/3.141592654)*(mapHeight-1));                   

           phiXY=phiXY/6.283185307+mapMidPoint;
           if(phiXY>=1){phiXY--;}
           if(phiXY<0){phiXY++;}
           mapX = (int)(phiXY*(mapWidth-1));

           colorInteger = 16777216 + mImage.getRGB(mapX,mapY);
           red = (int)(colorInteger/65536);
           green = (int)((colorInteger-red*65536)/256);
           blue = colorInteger-65536*red-256*green;      
           color = new Color(red, green, blue);
           spOriginalColor[i] = color;
       }	    
   }

   /**Calculates the brightness of each Pixel on this sphere given a light source.*/
   public void lumination (Pixel lumPoint, Color sourceColor)
   {
       int i;
       float lumDotProduct;
       double incidentDeviation, intoEyesEnergy;
       Pixel reflectedVector; 
       Pixel viewer = new Pixel(1,0,0);
       int sourceRed, sourceBlue, sourceGreen, newRed, newBlue, newGreen;
       Color newColor;

       sourceRed = sourceColor.getRed();
       sourceBlue = sourceColor.getBlue();
       sourceGreen = sourceColor.getGreen();

       for (i=0; i<pixCount; i++)
       {
           reflectedVector = Pixel.reflectedVector(spPixel[i], lumPoint);
           incidentDeviation = Math.acos(Pixel.unitDotProduct(reflectedVector, viewer));                   
           lumDotProduct= Pixel.unitDotProduct(lumPoint,spPixel[i]);
         
           if(lumDotProduct>=0.00)
           {  
               if(incidentDeviation<1.570796327)
               {		
                   intoEyesEnergy =Math.exp(-beta*incidentDeviation*beta*incidentDeviation);
               }
               else
               {intoEyesEnergy =0.0d;}

               lumDotProduct=(float)(0.9*(lumDotProduct+0.1)); 
               newRed = (int)(spOriginalColor[i].getRed()*(1-reflectivity)*lumDotProduct
             	             +sourceRed*reflectivity*intoEyesEnergy*lumDotProduct);
               newBlue= (int)(spOriginalColor[i].getBlue()*(1-reflectivity)*lumDotProduct
                             + sourceBlue*reflectivity*intoEyesEnergy*lumDotProduct);
               newGreen=(int)(spOriginalColor[i].getGreen()*(1-reflectivity)*lumDotProduct
                             + sourceGreen*reflectivity*intoEyesEnergy*lumDotProduct);
               newColor = new Color(newRed, newGreen, newBlue);
               spPixel[i].setPixelColor(newColor);
           }
           else
           {
               lumDotProduct=0.09f; 
               newRed = (int)(spOriginalColor[i].getRed()*0.01);
               newBlue= (int)(spOriginalColor[i].getBlue()*0.01);
               newGreen=(int)(spOriginalColor[i].getGreen()*0.01);

               newColor = new Color(newRed, newGreen, newBlue);
               spPixel[i].setPixelColor(newColor);
           }
       }
   }  
   
   /**Returns the number of Pixel on this sphere.*/
   public int getPixelCount()   { return pixCount; }

   /**Returns the width of the 2D map*/
   public int getImageWidth()   { return mImage.getWidth();   }

   /**Returns the height of the 2D map*/
   public int getImageHeight()   { return mImage.getHeight();  } 
}