import java.awt.*;

/**This class calculates the boundary of the 12 zodiac signs.*/
public class ZodiacMarker
{
   private static final double earthAXISCOS=0.917060074;//cos(23.5)
   private static final double earthAXISSIN=0.398749068; //sin(23.5)
   public Pixel[] marker = new Pixel[12];
   public Pixel[] originalMarker = new Pixel[12];
   private static int obsDistanceToEarth;
   private static int celestialRadius=180;
   private static int screenDistance=1000;
   public int noOfPixel=12;

   private double latRotCos, latRotSin, viewAngleCos, viewAngleSin;

   /**Costructor.*/
   public ZodiacMarker(int obs_Distance, double viewAngle, double latRot)
   {  
      int i;
      double angleStep, x, y, z;
      float brightness;

      viewAngleCos = Math.cos(viewAngle);
      viewAngleSin = Math.sin(viewAngle);
      latRotCos = Math.cos(latRot);
      latRotSin = Math.sin(latRot);           

      angleStep = 6.283185308/12; 
      obsDistanceToEarth=obs_Distance;

      for(i=0;i<noOfPixel;i++)
      {                
          x = celestialRadius*Math.sin(angleStep*i);
          y = celestialRadius*earthAXISCOS*Math.cos(angleStep*i);
          z = celestialRadius*earthAXISSIN*Math.cos(angleStep*i);

          brightness = (float)(0.5+0.25*(1+x/celestialRadius));
          originalMarker[i]=new Pixel((int)x,(int)y,(int)z);
      }
   }

   /**Calculates the boundary of the zodiac signs for a particular perspective.*/
   public void rotateZodiacMarker(double angle)					  
   {  
      double x, y, z, x1,y1,z1, cosRotation, sinRotation;

      cosRotation = Math.cos(angle);
      sinRotation = Math.sin(angle);

      for(int i=0;i<noOfPixel;i++)
      {
          x1 = cosRotation*originalMarker[i].getX()+sinRotation*originalMarker[i].getY();
          y1 =-sinRotation*originalMarker[i].getX()+cosRotation*originalMarker[i].getY();
          z1 = originalMarker[i].getZ();

          x = viewAngleCos*x1 + viewAngleSin*latRotSin*y1+viewAngleSin*latRotCos*z1;
          y = latRotCos*y1 - latRotSin*z1;                                         
          z = -viewAngleSin*x1 + viewAngleCos*latRotSin*y1+viewAngleCos*latRotCos*z1;

          y = y*screenDistance/(obsDistanceToEarth-x);
          z = z*screenDistance/(obsDistanceToEarth-x);

          marker[i] = new Pixel((int)x, (int)y, (int)z);                   
      }                         
   }

   /**Sets the latitude of the observer.*/
   public void setLatitude(double latRot)
   {
      latRotCos = Math.cos(latRot);
      latRotSin = Math.sin(latRot);	
   }

   /**Sets the perspective of the observer.*/
   public void setViewAngle(double viewAngle)
   {
      viewAngleCos = Math.cos(viewAngle);
      viewAngleSin = Math.sin(viewAngle);
   }
}