import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**This applet simulates the motion of sun w.r.t the earth in the course of a year.*/
public class SunInEarthPoint extends Applet
{  
   public SunEarthMovie movieFrame;
   public static MonthPanel monthPanel;
   public AngleSlider angleSlider;
   public DayPanel dayPanel; 
   public SunEarthControlPanel earthControlPanel;
   public PicturePanel title;
  
   /**Starts this applet by initializing all panels and processes needed in this applet.*/
   public void init()
   {
      setBackground(Color.black);
      setLayout(null);

      movieFrame = new SunEarthMovie(300,300);
      monthPanel = new MonthPanel(56,378);
      angleSlider = new AngleSlider();
      dayPanel = new DayPanel();
      earthControlPanel = new SunEarthControlPanel(523,50,150,280);
      title= new PicturePanel("Title.jpg");
	 
      add(monthPanel);
      add(angleSlider);
      add(movieFrame); 
      add(dayPanel);
      add(earthControlPanel);
      add(title);

      setVisible(true);

      monthPanel.setBounds(0,60,56,378);
      angleSlider.setBounds(0, 434,522,56);
      earthControlPanel.setBounds(0,490,523,75);
      movieFrame.setBounds(123,75,300,300);  
      dayPanel.setBounds(88,408,400,30);
      title.setBounds(0,0,523,60);

      movieFrame.start();
      dayPanel.start();
   }   
}

/**This window displays the motion of the sun w.r.t the earth in the course of a year.*/
class SunEarthMovie extends Canvas implements Runnable
{  
   private static int noPix;
   public MappedSphere earth= new MappedSphere(30);
   public  static double sunAngularSpeed, earthAngularSpeed;
   private final static int obsDistanceToEarth=700; 
   private static int sunR=0; 
   private static int sunY_Observed, sunZ_Observed, sunX=-1, earthX=0;
   private static int orbitRadius,celestXYelipHeight;
   private static Pixel lum=new Pixel(-100,0,0);
   private static final int celestialRadius=170; 
   private static final double screenDistance = 500;
   private static final int staticSunRadius=20;
   private static int canvasCenterX,canvasCenterY;
   public static boolean newObsAngleDetector=false;
   public static boolean freezingDetector=false;
   private Image mImage;
   public static double oldMonth, oneMonth;
  
   public static double absoluteTime;
   public static double obsAngleToEarth;//In radian
   public static double timeStep=2.5d;   
    
   private static Thread SunMotion = null;
   
   /**Constructor.*/
   public SunEarthMovie(int sizeX, int sizeY)
   {
       canvasCenterX=sizeX/2;
       canvasCenterY=sizeY/2;
       setBackground(Color.black); 
       setSize(sizeX, sizeY);                
   }
  
   /**Starts the simulation.*/
   public void start() {
                          SunMotion = new Thread(this, "SunEarthMovie");
                          SunMotion.start();
                       }

   /**Calculates the positions of the sun w.r.t the earth at a particular time in a year.*/
   public void run()
   {
       double x, y, z, startingTime,thirteenMonth;

       obsAngleToEarth=0;
   
       startingTime=-3*StandardConstLib.getQuarterYear()
                    +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3
                    -0.230136986*StandardConstLib.getQuarterYear();

       sunAngularSpeed= StandardConstLib.sunAngularSpeed;
       earthAngularSpeed=StandardConstLib.earthAngularSpeed;
          
       oneMonth = StandardConstLib.getQuarterYear()/3;            
       oldMonth = startingTime;
       thirteenMonth = 1.103196347*StandardConstLib.getQuarterYear();

       earth.createMap("world1.jpg");   
       earth.mappingXZrotation(obsAngleToEarth,0.75);
         
       noPix = earth.getPixelCount();

       TransformationMachine Tran = new TransformationMachine(obsAngleToEarth, 			obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);
           
       orbitRadius=(int)Tran.getOrbitRadius();
       celestXYelipHeight=(int)Tran.getcelestXYelipHeight();
           
       for(absoluteTime=startingTime;absoluteTime<=150000000;absoluteTime+=timeStep)	
       {    
          if((absoluteTime-oldMonth)>oneMonth)
	  {
              if(absoluteTime>thirteenMonth)
              {
                 SunInEarthPoint.monthPanel.setValue(0);
                 absoluteTime-=4*StandardConstLib.getQuarterYear();
              }
              else
              {
                  SunInEarthPoint.monthPanel.setValue(SunInEarthPoint.monthPanel.buttonChoice+1);
              }
              oldMonth=-3*StandardConstLib.getQuarterYear()
                       +(SunInEarthPoint.monthPanel.buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                       -0.230136986*StandardConstLib.getQuarterYear();
          }       

          Tran.transform(absoluteTime);
            	   
          x = Tran.getSunX();             	   
          y = Tran.getSunY();
          z = Tran.getSunZ();

          x= celestialRadius*x;
          y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
          z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
          sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 
                   
          lum=new Pixel((int)x,(int)y,(int)z);
          earth.lumination(lum);
                   
          sunX=(int)x;
          sunY_Observed = canvasCenterX -sunR/2 + (int)y;
          sunZ_Observed = canvasCenterY -sunR/2 - (int)z;
                
          render(); 
                   
          if (newObsAngleDetector)
          {
              Tran =new TransformationMachine(obsAngleToEarth,obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);
              orbitRadius=(int)Tran.getOrbitRadius();
              celestXYelipHeight=(int)Tran.getcelestXYelipHeight();
              earth.mappingXZrotation(obsAngleToEarth,0.75);

              newObsAngleDetector=false;
          }
                
          while(freezingDetector)
          {

              if (newObsAngleDetector)
              {
                 Tran =new TransformationMachine(obsAngleToEarth,obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);
                 orbitRadius=(int)Tran.getOrbitRadius();
                 celestXYelipHeight=(int)Tran.getcelestXYelipHeight();
                 earth.mappingXZrotation(obsAngleToEarth,0.75);

                 newObsAngleDetector=false;
              }

              Tran.transform(absoluteTime);
            	   
              x = Tran.getSunX();             	   
              y = Tran.getSunY();
              z = Tran.getSunZ();

              x= celestialRadius*x;
              y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
              z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
              sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 
                                
              lum=new Pixel((int)x,(int)y,(int)z);
              earth.lumination(lum);
                   
              sunX=(int)x;
              sunY_Observed = canvasCenterX -sunR/2 + (int)y;
              sunZ_Observed = canvasCenterY -sunR/2 - (int)z;
                
              render();
          } 
       }             
   }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
          Dimension d= getSize();
          if (checkImage(d))
          {
             Graphics imageGraphics = mImage.getGraphics();
             imageGraphics.setColor(getBackground());
             imageGraphics.fillRect(0,0,d.width,d.height);
             paint(imageGraphics);
             g.drawImage(mImage,0,0,null);
             imageGraphics.dispose();
          }
          g.dispose();
       }
   }   

   /**Paints the sun-earth system.*/
   public void paint(Graphics g)
   {   int i;
       float brightness;
       Color color;           
       Color c;
          
       if(obsAngleToEarth>=0)
       {
          g.setColor(Color.darkGray);
	  g.drawArc(canvasCenterX-orbitRadius,canvasCenterY-celestXYelipHeight,2*orbitRadius, 2*celestXYelipHeight,0,180);
       }
       else
       {
          g.setColor(Color.darkGray);
          g.drawArc(canvasCenterX-orbitRadius,canvasCenterY-celestXYelipHeight,2*orbitRadius, 2*celestXYelipHeight,180,180);
       }
    
       if(sunX <0)
       {
          g.setColor(Color.yellow);
          g.fillOval(sunY_Observed, sunZ_Observed, sunR, sunR);
          
          for(i=0; i<noPix;i++)
          {   
              brightness=earth.spPixel[i].getPixelBrightness();
              color=earth.spPixel[i].getPixelColor();
              c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
              g.setColor(c);
              g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
          }      
       }
       else
       { 
          for(i=0; i<noPix;i++)
          {   
              brightness=earth.spPixel[i].getPixelBrightness();
              color=earth.spPixel[i].getPixelColor();
              c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
                   g.setColor(c);
    		   g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
          }
          g.setColor(Color.yellow);
          g.fillOval(sunY_Observed,sunZ_Observed, sunR, sunR);
       }

       if(obsAngleToEarth>=0)
       {
          g.setColor(Color.white);
          g.drawArc(canvasCenterX-orbitRadius,canvasCenterY-celestXYelipHeight,2*orbitRadius, 2*celestXYelipHeight,180,180);
       }
       else
       {
          g.setColor(Color.white);
          g.drawArc(canvasCenterX-orbitRadius,canvasCenterY-celestXYelipHeight,2*orbitRadius, 2*celestXYelipHeight,0,180);
       }

       g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius, 2*orbitRadius,2*orbitRadius);
   }

   protected boolean checkImage(Dimension d)
   {
      if (d.width==0 || d.height == 0) return false;
      if (mImage == null || mImage.getWidth(null) != d.width || mImage.getHeight(null) 		!=d.height)
      {
          mImage=createImage(d.width,d.height);   
      }
      return true;
   }
}

/**This panel displays the user interface that allows users to select any particular month in a year.*/ 
class MonthPanel extends Panel implements MouseListener, MouseMotionListener
{
    private BufferedImage background;
    private BufferedImage[] button = new BufferedImage[12];
    private Image mImageBackground;
    public int mouseX1, mouseY1;
    private int buttonX=0, buttonY=332, buttonWidth=50, buttonHeight=20, buttonYDis=24;
    public int buttonChoice = 0;
    private String[] file = {"Mth01.jpg","Mth02.jpg","Mth03.jpg","Mth04.jpg","Mth05.jpg","Mth06.jpg","Mth07.jpg","Mth08.jpg","Mth09.jpg","Mth10.jpg","Mth11.jpg","Mth12.jpg"};
    private int panelHeight, panelWidth;
    private double quarterYear;

    /**Constructor.*/
    public MonthPanel(int sizeX, int sizeY)
    {
        int i;

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        try{
               InputStream in =getClass().getResourceAsStream("MonthButton.jpg");
               JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
               background = decoder.decodeAsBufferedImage();
               in.close(); 
           }catch(Exception e){} 

        panelWidth = sizeX;
        panelHeight = sizeY;
        addMouseListener(this);
        addMouseMotionListener(this);

        setValue(StandardConstLib.startingMonth-1);

        quarterYear=StandardConstLib.getQuarterYear();
           
        SunEarthMovie.absoluteTime =-3*quarterYear+quarterYear*StandardConstLib.startingMonth/3.00d
                                          -0.230136986*quarterYear;//0.230136986 = 21days
    }

    /**Sets the month of the year and repaints the user interface.*/
    public void setValue(int value)
    {
        buttonChoice = value;
        render();
    }

    /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
    protected void render()
    {
        Graphics g=getGraphics();
        if(g!=null)
        {
            mImageBackground=createImage(panelWidth, panelHeight);
            Graphics imageGraphics = mImageBackground.getGraphics();
            paint(imageGraphics);
            g.drawImage(mImageBackground,0,0,null);
            imageGraphics.dispose();
            g.dispose();
        }
    }     

    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        int i;
        g.drawImage(background,0,0,this);
        g.drawImage(button[buttonChoice],buttonX, buttonY-(buttonChoice+1)*buttonYDis,this);
    }

    /**Resets the month of the year if a particular button corresponding to that month is pressed.*/
    public void mousePressed(MouseEvent e)
    {
        double temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if(mouseX1>buttonX&&mouseX1<buttonX+buttonWidth)
        {
            temp = (double)(buttonY-mouseY1)/buttonYDis;
            if((temp-(int)temp<(double)buttonHeight/buttonYDis)&&buttonChoice!=(int)temp)
            {
                buttonChoice = (int)temp;

                SunEarthMovie.absoluteTime=-3*quarterYear+quarterYear*(double)(buttonChoice+1)/3.00d
                                           -0.230136986*quarterYear;
                           
                SunEarthMovie.oldMonth=-3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                                       -0.230136986*StandardConstLib.getQuarterYear();
                render();
            }
        }
    }

    public void mouseReleased(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}

    public void mouseDragged(MouseEvent e){}

    public void mouseMoved(MouseEvent e){}
}

/**This class contains user interface that allows users to freeze, unfreeze, slow down and speed up the applet.*/
class SunEarthControlPanel extends Canvas implements MouseListener
{
    private BufferedImage background;
    private BufferedImage[] speedImage = new BufferedImage[4];
    private BufferedImage[] timeImage = new BufferedImage[4];
    private Image mImage;
    private String[] speedPicName = {"speedL.jpg","speedN.jpg","speedMinusB.jpg","speedPlusB.jpg"};
    private String[] timePicName = {"freezetime1.jpg","freezetime2.jpg","unfreezetime1.jpg","unfreezetime2.jpg"};
    private int timeX = 0, timeY = 8;
    private int speedX = 145, speedY = 8;   
    private int speedW = 112, speedH =37;
    private int timeW = 121, timeH =37;
    private int height, width;

    public static int speedNo = 1, timeNo = 0, resetNo = 0;

    /**Constructor.*/
    public SunEarthControlPanel(int sizeX, int sizeY, int timePos, int speedPos)
    {
        timeX=timePos;
        speedX=speedPos;

        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(speedPicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   speedImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }
   
        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(timePicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   timeImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }

        width=sizeX;
        height=sizeY;
        setBackground(new Color(6,76,73));
        addMouseListener(this);
        setVisible(true);
    }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImage=createImage(width, height);   
           Graphics imageGraphics = mImage.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImage,0,0,this);
           imageGraphics.dispose();
           g.dispose();
       }
   }
     
    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        g.drawImage(speedImage[speedNo], speedX, speedY, this);
        g.drawImage(timeImage[timeNo], timeX, timeY, this);
    }

    /**Detects and records the button that is pressed.*/
    public void mousePressed(MouseEvent e)
    {
       int mouseX, mouseY;
       mouseX = e.getX();
       mouseY = e.getY();

       if((mouseX>timeX)&&(mouseX<timeX+timeW)&&(mouseY>timeY)&&(mouseY<timeY+timeH))
       {
           timeNo+=1;
       }
       
       if(timeNo==0)
       {
           if((mouseX>speedX+3)&&(mouseX<speedX+56)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=3;
           }
       
           if((mouseX>speedX+61)&&(mouseX<speedX+111)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=2;
           }
       }
       render();
    }

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
 
    /**Resets the state of the applet when when a button that was previous pressed on is now released.*/
    public void mouseReleased(MouseEvent e)
    {
        if(timeNo==1)
        {
            timeNo=2; speedNo=0;
            SunEarthMovie.freezingDetector=true;
        }
        else if(timeNo==3)
        {
            timeNo=0; speedNo=1;
            SunEarthMovie.freezingDetector=false;
        }

        if(speedNo==2)
        {
            speedNo=1;
            if(SunEarthMovie.timeStep>0.51)
            {SunEarthMovie.timeStep-=0.251;}
            else
            { SunEarthMovie.timeStep=SunEarthMovie.timeStep/2;}
        }

        if(speedNo==3)
        {   
            speedNo=1;         
            if(SunEarthMovie.timeStep<15&&SunEarthMovie.timeStep>=0.251)
            { 
                SunEarthMovie.timeStep+=0.251;
            }
            else if(SunEarthMovie.timeStep<0.251)
            {SunEarthMovie.timeStep*=2;}
        }

        render();
    }
}

/**A panel that contains user inteface for changing the viewing angle of the sun-earth system*/
class AngleSlider extends Panel implements MouseListener, MouseMotionListener
{
   private BufferedImage slider, pointer;
   private Image mImageBackground;
   public int mouseX1, mouseY1;
   public int mouseX2, mouseY2;
   private int angleValue = 0, newAngleValue;
   private boolean onSliderKnob = false;
   private int sliderX=126, sliderY=27,sliderWidth=300;

   /**Constructor*/
   public AngleSlider()
   {
    
      try{
           InputStream in =getClass().getResourceAsStream("AngleSlider.jpg");
           JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
           slider = decoder.decodeAsBufferedImage();
           in.close(); 
         }catch(Exception e){}    

      try{
           InputStream in =getClass().getResourceAsStream("Marker.jpg");
           JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
           pointer = decoder.decodeAsBufferedImage();
           in.close(); 
         }catch(Exception e){} 

      SunEarthMovie.obsAngleToEarth=StandardConstLib.obsAngleToEarth*3.141592654/180;

      addMouseListener(this);
      addMouseMotionListener(this);
   }

   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImageBackground=createImage(slider.getWidth(), slider.getHeight());   
           Graphics imageGraphics = mImageBackground.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImageBackground,0,0,null);
           imageGraphics.dispose();
           g.dispose();
       }
   }     


   /**Draws the interface.*/
   public void paint(Graphics g)
   {            
      g.drawImage(slider,0,0, this);
      g.drawImage(pointer,sliderX+sliderWidth*(90-angleValue)/180-14, sliderY, this);
   }


   /**Detects if the mouse pointer is on the knob of the slider when a mouse button is pressed.*/
   public void mousePressed(MouseEvent e)
   {
      mouseY1 = e.getY();
      mouseX1 = e.getX();

      if(mouseY1<sliderY+15 && mouseY1>sliderY && mouseX1>(sliderX+sliderWidth*(90-angleValue)/180-11)&& mouseX1<(sliderX+sliderWidth*(90-angleValue)/180+11) )
      { onSliderKnob = true;}

   }

   public void mouseReleased(MouseEvent e){onSliderKnob = false;}

   public void mouseClicked(MouseEvent e){}

   public void mouseEntered(MouseEvent e){}

   public void mouseExited(MouseEvent e){onSliderKnob = false;}

   /**Passes the new viewing angle to SunEarthMovie if new viewing angle is set.*/
   public void mouseDragged(MouseEvent e)
   {
      mouseX2 = e.getX();
      mouseY2 = e.getY();

      if(onSliderKnob)
      {
         if(mouseX2>sliderX-5&&mouseX2<sliderX+sliderWidth+5)
         {
            newAngleValue = 90-(mouseX2-sliderX)*180/sliderWidth;
            if(newAngleValue!=angleValue)
            {
               angleValue = newAngleValue;
               if(angleValue>90){newAngleValue=90;angleValue=90;}
               if(angleValue<-90){newAngleValue=-90;angleValue=-90;}
               SunEarthMovie.obsAngleToEarth = angleValue*3.141592654/180;
               SunEarthMovie.newObsAngleDetector=true;
               render();
            }
         }
      }
   }

   public void mouseMoved(MouseEvent e){}
}


/**This panel tells users the day of a year that the motion on the screen correspond to.*/
class DayPanel extends Panel implements Runnable
{
    private Thread dayTimer;       
    private int dayOfTheYear, newDay=61;
    private static String[] numberString={"0","1","2","3","4","5","6","7","8","9"};

    //comments:absoluteTime=0 at September Equinox =264.33days in one year.
    
   /**Constructor.*/
    public DayPanel()
    {
       setSize(100,20);
    }
   
   /**Starts the panel.*/
    public void start()
    {
       dayTimer = new Thread(this, "DayPanel");
       dayTimer.start();
    }  

   /**Calculates the day of the year.*/
    public void run()
    {
       dayOfTheYear= ((StandardConstLib.startingMonth-1)*365)/12;
       newDay=dayOfTheYear;

       while(true)
       {   
           newDay = (int)(SunEarthMovie.absoluteTime*StandardConstLib.sunAngularSpeed*58.09155423+264.33334d+1);
           if(newDay!=dayOfTheYear)
           {
               repaint();
               dayOfTheYear=newDay;
           }

           try{
                  Thread.sleep(300);
              }catch(InterruptedException e){} 
       }
    }
	
   /**Paints the day of the year on the screen.*/
    public void paint(Graphics g)
    {
       int newDayCounter;
       double angle;
       float floatAngle;
       String finalString;

       g.setColor(Color.cyan);
            
       newDayCounter=newDay;
             
       finalString=Integer.toString(newDayCounter);
       finalString=finalString.concat(" of 365 days");
       g.drawString(finalString, 35,10);
            
       angle = -57.2957795*Math.asin(0.398749068*Math.sin(StandardConstLib.sunAngularSpeed*SunEarthMovie.absoluteTime));
	
       finalString = "Declination angle of the Sun = ";
            
       floatAngle = (float)((int)(angle*10))/10;	    
       finalString = finalString.concat(Float.toString(floatAngle));
            
       g.drawString( finalString, 150,10);
    }
}


/**This class stores important constants that are used in this applet.*/
class StandardConstLib

{    
    public final static int startingMonth=3;
    public final static double obsAngleToEarth=0.00d;
    public final static double sunAngularSpeed=0.00008291, earthAngularSpeed=0.03034506;
    
    public static double getQuarterYear()
    {
       return 1.570796327/sunAngularSpeed;
    }
}