import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**This applet simulates the motion of sun w.r.t observers at different latitudes in the course of a year.*/
public class SunAtDifferentLatitude2 extends Applet
{  
   public SunEarthMovie movieFrame;
   public static MonthPanel monthPanel;
   public DayPanel dayPanel; 
   public SunEarthControlPanel latitudeControlPanel;
   public MapDetectorPanel mapDetectorPanel;
   public PicturePanel title, click;


   /**Starts this applet by initializing all panels and processes needed in this applet.*/
   public void init()
   { 
       setBackground(Color.black);   
       setLayout(null);
          
       movieFrame = new SunEarthMovie(300,300);
       dayPanel = new DayPanel();
       monthPanel = new MonthPanel(56,378);
       latitudeControlPanel = new SunEarthControlPanel(525,50,117,255);
       title= new PicturePanel("Title.jpg");
       click= new PicturePanel("clickMap.jpg");
       mapDetectorPanel = new MapDetectorPanel("world1.jpg");

       add(monthPanel);
       add(movieFrame); 
       add(dayPanel);
       add(latitudeControlPanel);
       add(mapDetectorPanel);   
       add(title);
       add(click);

       setVisible(true);
     
       latitudeControlPanel.setBounds(0,434,750,116);
       monthPanel.setBounds(0,60,56,378);
       movieFrame.setBounds(90,85,300,300);
       dayPanel.setBounds(83,408,400,22);
       mapDetectorPanel.setBounds(430,165,306,152);
       click.setBounds(430,317,306,30);
       title.setBounds(0,0,750,60);

       movieFrame.start();   
       dayPanel.start();
   } 
}

/**This window displays the motion of the sun w.r.t an observer at a particular latitude in the course of a year.*/
class SunEarthMovie extends Canvas implements Runnable
{  
   private static int noPix;
   private int topAxisX2, topAxisY2; 
   private double axisDirectionY, axisDirectionZ;
   MappedSphere earth= new MappedSphere(32);
   public  static double sunAngularSpeed, earthAngularSpeed;
   private final static int obsDistanceToEarth=700; 
   private static int sunR=0; 
   private static int sunY_Observed, sunZ_Observed, sunX=-1, earthX=0;
   private static int orbitRadius;
   private static Pixel lum=new Pixel(-100,0,0);
   private static final int celestialRadius=170; 
   private static final double screenDistance = 500;
   private static final int staticSunRadius=20;
   private static int canvasCenterX,canvasCenterY;
   public static boolean newLatitudeDetector=false;
   public static boolean freezingDetector=false;
   private Image mImage;
   public static double oldMonth, oneMonth, oneDay;
   public static Pixel planeDirection= new Pixel(0,0,1);
  
   public static double absoluteTime;
   public static double latitude;//In radian
   public static double longitude=0.5;//In radian
   public static double timeStep=2.5d;
   public static double phase =0.0d;   
    
   private static Thread SunMotion = null;
   
   /**Constructor.*/
   public SunEarthMovie(int sizeX, int sizeY)
   {
       canvasCenterX=sizeX/2;
       canvasCenterY=sizeY/2;
       setBackground(Color.black); 
       setSize(sizeX, sizeY);
   }
 
   /**Starts the simulation.*/
   public void start() {
                      SunMotion = new Thread(this, "SunEarthMovie");
                      SunMotion.start();
                       }

   /**Calculates the positions of the sun w.r.t an observer at a particular latitude at a particular time in a year.*/
   public void run()
   {  
       double x, y, z, startingTime,thirteenMonth;

       latitude=StandardConstLib.latitude*3.141592654/180;
       longitude = StandardConstLib.longitude;
   
       startingTime=-3*StandardConstLib.getQuarterYear()
	            +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3
                    -0.230136986*StandardConstLib.getQuarterYear();

       sunAngularSpeed= StandardConstLib.sunAngularSpeed;
       earthAngularSpeed=StandardConstLib.earthAngularSpeed;
       DayPanel.sinRotation = Math.sin(1.570796327-latitude);
       DayPanel.cosRotation = Math.cos(1.570796327-latitude);
          
       oneMonth = StandardConstLib.getQuarterYear()/3;            
       oldMonth = startingTime;
       oneDay = 6.283185307/StandardConstLib.earthAngularSpeed;
       phase = longitude*oneDay;

       thirteenMonth = 1.103196347*StandardConstLib.getQuarterYear();
       earth.createMap("world1.jpg");  
       earth.mappingYZrotation(1.570796327-latitude,longitude);          
       noPix = earth.getPixelCount();
         
       TransformationMachine Tran = new TransformationMachine(1.570796327-latitude,obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);

       orbitRadius=(int)Tran.getOrbitRadius();
          
       axisDirectionY=-Math.cos(latitude);
       axisDirectionZ=Math.sin(latitude);
             
       topAxisX2 = (int)(orbitRadius*axisDirectionY);   
       topAxisY2 = (int)(orbitRadius*axisDirectionZ);     
           
       for(absoluteTime=startingTime;absoluteTime<=1500000;absoluteTime+=timeStep)
       {           
           if((absoluteTime-oldMonth)>oneMonth)
           {
              if(absoluteTime>thirteenMonth)
              {
                 SunAtDifferentLatitude2.monthPanel.setValue(0);
                 absoluteTime-=4*StandardConstLib.getQuarterYear();
              }
              else
              {
                  SunAtDifferentLatitude2.monthPanel.setValue(SunAtDifferentLatitude2.monthPanel.buttonChoice+1);
              }
              oldMonth=-3*StandardConstLib.getQuarterYear()
                       +(SunAtDifferentLatitude2.monthPanel.buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                       -0.230136986*StandardConstLib.getQuarterYear();
           }       

           Tran.transform2(absoluteTime+phase);
   
           x = Tran.getSunX();             	   
           y = Tran.getSunY();
           z = Tran.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
           sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

           lum=new Pixel((int)x,(int)y,(int)z);
           earth.lumination(lum);
           
           sunX=(int)x;
           sunY_Observed = canvasCenterX -sunR/2 + (int)y;
           sunZ_Observed = canvasCenterY -sunR/2 - (int)z;     
           render();
                   
           if (newLatitudeDetector)
           {
               Tran =new TransformationMachine(1.570796327-latitude,obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);
               DayPanel.sinRotation = Math.sin(1.570796327-latitude);
               DayPanel.cosRotation = Math.cos(1.570796327-latitude);

               orbitRadius=(int)Tran.getOrbitRadius();
               earth.mappingYZrotation(1.570796327-latitude, longitude);
               phase = longitude*oneDay;
               axisDirectionY=-Math.cos(latitude);
               axisDirectionZ=Math.sin(latitude);

               topAxisX2 = (int)(orbitRadius*axisDirectionY);   
               topAxisY2 = (int)(orbitRadius*axisDirectionZ);

               newLatitudeDetector=false;
           }
                
           while(freezingDetector)
           {
               if (newLatitudeDetector)
               {
                  Tran =new TransformationMachine(1.570796327-latitude, 
                  obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);

                  DayPanel.sinRotation = Math.sin(1.570796327-latitude);
                  DayPanel.cosRotation = Math.cos(1.570796327-latitude);

                  orbitRadius=(int)Tran.getOrbitRadius();	
                  earth.mappingYZrotation(1.570796327-latitude,longitude);
                  phase = longitude*oneDay;
			
                  axisDirectionY=-Math.cos(latitude);
                  axisDirectionZ=Math.sin(latitude);

                  topAxisX2 = (int)(orbitRadius*axisDirectionY);   
                  topAxisY2 = (int)(orbitRadius*axisDirectionZ);
                  newLatitudeDetector=false;
               }

               Tran.transform2(absoluteTime+phase);
               x = Tran.getSunX();             	   
               y = Tran.getSunY();
               z = Tran.getSunZ();

               x= celestialRadius*x;
               y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
               z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
               sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius);  

               lum=new Pixel((int)x,(int)y,(int)z);
               earth.lumination(lum);
           
               sunX=(int)x;
               sunY_Observed = canvasCenterX -sunR/2 + (int)y;
               sunZ_Observed = canvasCenterY -sunR/2 - (int)z;
                
               render();
          }
       }
   }
   
   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           Dimension d= getSize();
           if (checkImage(d))
           {
                Graphics imageGraphics = mImage.getGraphics();
                imageGraphics.setColor(getBackground());
                imageGraphics.fillRect(0,0,d.width,d.height);
                paint(imageGraphics);
                g.drawImage(mImage,0,0,null);
                imageGraphics.dispose();
           }
           g.dispose();
       }
   }   

   /**Paints the sun-earth system to an observer at a particular latitude.*/
   public void paint(Graphics g)
   {
       int i, planeRadius=5;
       float brightness, planeBrightness;
       Color color;           
       Color c, planeColor;
 
       planeBrightness=Pixel.unitDotProduct(lum, planeDirection);
       
       if(planeBrightness<=0)
       {planeColor=new Color(110,60,30);}
       else  {planeColor=new Color(255,220,180);}
          
       g.setColor(Color.white);
       g.drawLine(canvasCenterX-orbitRadius,canvasCenterY,canvasCenterX+orbitRadius,canvasCenterY);
       g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius, 2*orbitRadius,2*orbitRadius);

       if(sunX <0)
       {
          g.setColor(Color.yellow);
          g.fillOval(sunY_Observed, sunZ_Observed, sunR, sunR);

          g.setColor(new Color(200,200,255));
	  g.drawLine(canvasCenterX+topAxisX2, canvasCenterY-topAxisY2,canvasCenterX-topAxisX2, canvasCenterY+topAxisY2);

          for(i=0; i<noPix;i++)
          {   
              brightness=earth.spPixel[i].getPixelBrightness();
              color=earth.spPixel[i].getPixelColor();
              c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
              g.setColor(c);
              g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(),1,1);
          }

	  g.setColor(planeColor);
          g.fillOval(canvasCenterX-planeRadius,canvasCenterY-31,2*planeRadius, planeRadius-1); 
       }
       else
       { 
          g.setColor(new Color(200,200,255));
          g.drawLine(canvasCenterX+topAxisX2, canvasCenterY-topAxisY2,canvasCenterX-topAxisX2, canvasCenterY+topAxisY2);

          for(i=0; i<noPix;i++)
          {   
              brightness=earth.spPixel[i].getPixelBrightness();
              color=earth.spPixel[i].getPixelColor();
              c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
              g.setColor(c);
              g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
          }

          g.setColor(planeColor);
          g.fillOval(canvasCenterX-planeRadius,canvasCenterY-31,2*planeRadius,planeRadius-1);

          g.setColor(Color.yellow);
          g.fillOval(sunY_Observed,sunZ_Observed, sunR, sunR);
       }
   }


   protected boolean checkImage(Dimension d)
   {
       if (d.width==0 || d.height == 0) return false;
       if (mImage == null || mImage.getWidth(null) != d.width || mImage.getHeight(null)!=d.height)
       {
          mImage=createImage(d.width,d.height);   
       }
       return true;
   }
}

/**This panel displays a world map and allows users to change the position of the observer by clicking on the world map.*/
class MapDetectorPanel extends Panel implements MouseListener
{
   BufferedImage mImage;
   Image mImageBackground;
   int imageWidth, imageHeight;
   int pointX, pointY;

   double longitudeRatio;    

    /**Constructor that creates a map in memory by opening a picture file.*/
   public MapDetectorPanel(String filename)
   {
      try{
           InputStream in =getClass().getResourceAsStream(filename);
           JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
           mImage= decoder.decodeAsBufferedImage();
           in.close(); 
         }catch(Exception e){}  
   
      setSize(mImage.getWidth(), mImage.getHeight()); 
      addMouseListener(this); 
      imageWidth = mImage.getWidth();
      imageHeight = mImage.getHeight();
      pointX = (int)(imageWidth*(StandardConstLib.longitude+0.2)); 
      pointY = (int)(imageHeight*(0.5-(double)StandardConstLib.latitude/180)); 
   }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
      Graphics g=getGraphics();
      if(g!=null)
      {
          mImageBackground=createImage(imageWidth, imageHeight);   
          Graphics imageGraphics = mImageBackground.getGraphics();
          paint(imageGraphics);
          g.drawImage(mImageBackground,0,0,null);
          imageGraphics.dispose();
          g.dispose();
      }
   }     

    /**Paints the map.*/
   public void paint(Graphics g)
   {
      g.drawImage(mImage, 0, 0, this);
      g.setColor(new Color(255,220,180));
      g.fillOval(pointX-5,pointY-3, 10, 5);
   }
   
   /**Sets the new position of the observer if a user selects a new position on the map*/
   public void mousePressed(MouseEvent e) 
   {
      SunEarthMovie.latitude = 3.141592654*(0.5-e.getY()/(double)imageHeight);
      longitudeRatio= e.getX()/(double)imageWidth;

      if(longitudeRatio<=0.2)           
      {SunEarthMovie.longitude = longitudeRatio+0.8d; }
      else {SunEarthMovie.longitude = longitudeRatio-0.2d; }

      SunEarthMovie.newLatitudeDetector=true; 

      DayPanel.newLatDetector=true;
      pointX = e.getX();
      pointY = e.getY();
      render();
   }

   public void mouseClicked(MouseEvent e) { }

   public void mouseEntered(MouseEvent e) { }

   public void mouseExited(MouseEvent e) { }

   public void mouseReleased(MouseEvent e) { }
}

/**This panel displays the user interface that allows users to select any particular month in a year.*/ 
class MonthPanel extends Panel implements MouseListener, MouseMotionListener
{
    private BufferedImage background;
    private BufferedImage[] button = new BufferedImage[12];
    private Image mImageBackground;
    public int mouseX1, mouseY1;
    private int buttonX=0, buttonY=332, buttonWidth=50, buttonHeight=20, buttonYDis=24;
    public int buttonChoice = 0;
    private String[] file = {"Mth01.jpg","Mth02.jpg","Mth03.jpg","Mth04.jpg","Mth05.jpg","Mth06.jpg","Mth07.jpg","Mth08.jpg","Mth09.jpg","Mth10.jpg","Mth11.jpg","Mth12.jpg"};
    private int panelHeight, panelWidth;
    private double quarterYear;

    /**Constructor.*/
    public MonthPanel(int sizeX, int sizeY)
    {
        int i;

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        try{
               InputStream in =getClass().getResourceAsStream("MonthButton.jpg");
               JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
               background = decoder.decodeAsBufferedImage();
               in.close(); 
           }catch(Exception e){} 

        panelWidth = sizeX;
        panelHeight = sizeY;
        addMouseListener(this);
        addMouseMotionListener(this);

        setValue(StandardConstLib.startingMonth-1);

        quarterYear=StandardConstLib.getQuarterYear();
           
        SunEarthMovie.absoluteTime =-3*quarterYear+quarterYear*StandardConstLib.startingMonth/3.00d
                                          -0.230136986*quarterYear;//0.230136986 = 21days
    }

    /**Sets the month of the year and repaints the user interface.*/
    public void setValue(int value)
    {
        buttonChoice = value;
        render();
    }

    /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
    protected void render()
    {
        Graphics g=getGraphics();
        if(g!=null)
        {
            mImageBackground=createImage(panelWidth, panelHeight);
            Graphics imageGraphics = mImageBackground.getGraphics();
            paint(imageGraphics);
            g.drawImage(mImageBackground,0,0,null);
            imageGraphics.dispose();
            g.dispose();
        }
    }     

    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        int i;
        g.drawImage(background,0,0,this);
        g.drawImage(button[buttonChoice],buttonX, buttonY-(buttonChoice+1)*buttonYDis,this);
    }

    /**Resets the month of the year if a particular button corresponding to that month is pressed.*/
    public void mousePressed(MouseEvent e)
    {
        double temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if(mouseX1>buttonX&&mouseX1<buttonX+buttonWidth)
        {
            temp = (double)(buttonY-mouseY1)/buttonYDis;
            if((temp-(int)temp<(double)buttonHeight/buttonYDis)&&buttonChoice!=(int)temp)
            {
                buttonChoice = (int)temp;

                SunEarthMovie.absoluteTime=-3*quarterYear+quarterYear*(double)(buttonChoice+1)/3.00d
                                           -0.230136986*quarterYear;
                           
                SunEarthMovie.oldMonth=-3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                                       -0.230136986*StandardConstLib.getQuarterYear();
                render();
            }
        }
    }

    public void mouseReleased(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}

    public void mouseDragged(MouseEvent e){}

    public void mouseMoved(MouseEvent e){}
}

/**This panel tells users the day of a year that the motion on the screen correspond to.*/
class DayPanel extends Panel implements Runnable
{
    private Thread dayTimer;       
    private int dayOfTheYear, newDay=61;
    public static double cosRotation=0.00d, sinRotation=0.00d, dayLength;
    private static String[] numberString={"0","1","2","3","4","5","6","7","8","9"};
    public static boolean newLatDetector=false;

    //comments:absoluteTime=0 at September Equinox =264.33days in one year.
    
   /**Constructor.*/
    public DayPanel()
    {
       setSize(100,20);
    }

   /**Starts the panel.*/
    public void start()
    {
       dayTimer = new Thread(this, "DayPanel");
       dayTimer.start();
    }  
 
   /**Calculates the day of the year and the length of that day*/
    public void run()
    {
       dayOfTheYear= ((StandardConstLib.startingMonth-1)*365)/12;
       newDay=dayOfTheYear; 

       while(true)
       {   
           newDay = (int)(SunEarthMovie.absoluteTime*StandardConstLib.sunAngularSpeed*58.09155423+264.33334d+1);

           if(newDay!=dayOfTheYear)
           {
               dayLength = dayTime(SunEarthMovie.absoluteTime);
               repaint();
               dayOfTheYear=newDay;
           }

           if(newLatDetector)
           {
               sinRotation =Math.sin(1.570796327-SunEarthMovie.latitude);
               cosRotation =Math.cos(1.570796327-SunEarthMovie.latitude);
               dayLength = dayTime(SunEarthMovie.absoluteTime);
               repaint();
           }

     	   try{
                  Thread.sleep(300);
              }catch(InterruptedException e){} 
       }
    }
	
   /**Paints the day of the year on the screen.*/
    public void paint(Graphics g)
    {
       int newDayCounter;
       String[] lengthString =  new String[3];
       String finalString;          

       g.setColor(Color.cyan);

       newDayCounter=newDay;
       finalString=Integer.toString(newDayCounter);
       finalString=finalString.concat(" of 365 days");
       g.drawString(finalString, 30,10);           

       dayLength = (float)((int)(dayLength*10))/10;
       lengthString[0]="Length of the day = ";
       lengthString[1]=Float.toString((float)dayLength);
       lengthString[2]=" hours";
            
       finalString = lengthString[0].concat(lengthString[1]);
       finalString = finalString.concat(lengthString[2]);
       g.drawString(finalString,130,10);
   }

   /**Returns the length of a particular day at a certain latitude.*/
   public double dayTime(double time)
   {
       double sinSun, cosSun;
       double maxHeightAboveHorizon, minHeightAboveHorizon;
       double dayTimeResult=0;
       double staticPosition, amplitude; 

       sinSun=Math.sin(StandardConstLib.sunAngularSpeed*time);
       cosSun=Math.cos(StandardConstLib.sunAngularSpeed*time);

       amplitude = sinRotation*Math.sqrt(0.84099918+0.15900082*cosSun*cosSun);
       staticPosition = -cosRotation*0.398749068*sinSun;

       maxHeightAboveHorizon = staticPosition +amplitude;
       minHeightAboveHorizon = staticPosition -amplitude;

       if (maxHeightAboveHorizon <=0)
       {dayTimeResult = 0.00d;}

       if (minHeightAboveHorizon >=0)
       {dayTimeResult = 24.00d;}
	    
       if ((maxHeightAboveHorizon > 0) && (minHeightAboveHorizon < 0))
       {
           dayTimeResult = 24.05*maxHeightAboveHorizon/(maxHeightAboveHorizon-minHeightAboveHorizon);
       } 

       return dayTimeResult;
   }
}

/**This class contains user interface that allows users to freeze, unfreeze, slow down and speed up the applet.*/
class SunEarthControlPanel extends Canvas implements MouseListener
{
    private BufferedImage background;
    private BufferedImage[] speedImage = new BufferedImage[4];
    private BufferedImage[] timeImage = new BufferedImage[4];
    private Image mImage;
    private String[] speedPicName = {"speedL.jpg","speedN.jpg","speedMinusB.jpg","speedPlusB.jpg"};
    private String[] timePicName = {"freezetime1.jpg","freezetime2.jpg","unfreezetime1.jpg","unfreezetime2.jpg"};
    private int timeX = 0, timeY = 12;
    private int speedX = 145, speedY = 12;   
    private int speedW = 112, speedH =37;
    private int timeW = 121, timeH =37;
    private int height, width;

    public static int speedNo = 1, timeNo = 0, resetNo = 0;

    /**Constructor.*/
    public SunEarthControlPanel(int sizeX, int sizeY, int timePos, int speedPos)
    {
        timeX=timePos;
        speedX=speedPos;

        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(speedPicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   speedImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }
   
        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(timePicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   timeImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }

        width=sizeX;
        height=sizeY;
        setBackground(new Color(6,76,73));
        addMouseListener(this);
        setVisible(true);
    }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImage=createImage(width, height);   
           Graphics imageGraphics = mImage.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImage,0,0,this);
           imageGraphics.dispose();
           g.dispose();
       }
   }
     
    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        g.setColor(new Color(75,75,75));
        g.fillRect(0,0,750,5);
        g.setColor(new Color(0,109,104));
        g.fillRect(0,5,750,50);
        g.setColor(new Color(4,76,73));
        g.fillRect(0,55,750,64);
        g.drawImage(speedImage[speedNo], speedX, speedY, this);
        g.drawImage(timeImage[timeNo], timeX, timeY, this);
    }

    /**Detects and records the button that is pressed.*/
    public void mousePressed(MouseEvent e)
    {
       int mouseX, mouseY;
       mouseX = e.getX();
       mouseY = e.getY();

       if((mouseX>timeX)&&(mouseX<timeX+timeW)&&(mouseY>timeY)&&(mouseY<timeY+timeH))
       {
           timeNo+=1;
       }
       
       if(timeNo==0)
       {
           if((mouseX>speedX+3)&&(mouseX<speedX+56)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=3;
           }
       
           if((mouseX>speedX+61)&&(mouseX<speedX+111)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=2;
           }
       }
       render();
    }

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
 
    /**Resets the state of the applet when when a button that was previous pressed on is now released.*/
    public void mouseReleased(MouseEvent e)
    {
        if(timeNo==1)
        {
            timeNo=2; speedNo=0;
            SunEarthMovie.freezingDetector=true;
        }
        else if(timeNo==3)
        {
            timeNo=0; speedNo=1;
            SunEarthMovie.freezingDetector=false;
        }

        if(speedNo==2)
        {
            speedNo=1;
            if(SunEarthMovie.timeStep>0.51)
            {SunEarthMovie.timeStep-=0.251;}
            else
            { SunEarthMovie.timeStep=SunEarthMovie.timeStep/2;}
        }

        if(speedNo==3)
        {   
            speedNo=1;         
            if(SunEarthMovie.timeStep<15&&SunEarthMovie.timeStep>=0.251)
            { 
                SunEarthMovie.timeStep+=0.251;
            }
            else if(SunEarthMovie.timeStep<0.251)
            {SunEarthMovie.timeStep*=2;}
        }

        render();
    }
}

/**This class stores important constants that are used in this applet.*/
class StandardConstLib
{    
    public final static int startingMonth=3;
    public final static double latitude=90.00d;
    public final static double longitude = 0.5;
    public final static double obsAngleToEarth=0.00d;
    public final static double sunAngularSpeed=0.00008291, earthAngularSpeed=0.03034506;
    public final static double moonAngularSpeed=0.001110659;
    
    public static double getQuarterYear()
    {
        return 1.570796327/sunAngularSpeed;
    }
}



