import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;

/**This applet simulates the motion of sun w.r.t observers at different latitudes on different fixed days in the year.*/
public class StaticTrack3 extends Applet
{  
   public SunEarthMovie movieFrame;
   public static MonthPanel monthPanel;
   public SunEarthControlPanel controlPanel;
   public static StaticDayPanel staticDayPanel;
   public MapDetectorPanel mapDetectorPanel;
   public PicturePanel title, click;

   /**Starts this applet by initializing all panels and processes needed in this applet.*/
   public void init()
   {
       setBackground(new Color(0,0,0));
       setLayout(null);

       movieFrame = new SunEarthMovie(340,320);
       controlPanel = new SunEarthControlPanel(525,50,75,215);
       title= new PicturePanel("Title.jpg");
       monthPanel = new MonthPanel(339,125);
       click= new PicturePanel("clickMap.jpg");
       staticDayPanel = new StaticDayPanel();
       mapDetectorPanel = new MapDetectorPanel("world1.jpg");

       add(monthPanel);
       add(movieFrame);
       add(staticDayPanel);
       add(controlPanel);
       add(mapDetectorPanel);   
       add(title);
       add(click);

       setVisible(true);    

       controlPanel.setBounds(0,434,750,116);
       monthPanel.setBounds(411,60,339,125);
       movieFrame.setBounds(30,65,340,340);
       staticDayPanel.setBounds(120,408,230,22);       
       mapDetectorPanel.setBounds(416,205,304,151);
       title.setBounds(0,0,750,60);
       click.setBounds(416,355,306,30);

       movieFrame.start(); 
   }
}

/**This window displays the motion of the sun w.r.t an observer at a particular latitude on different fixed days in the year.*/
class SunEarthMovie extends Canvas implements Runnable
{  
   private static int noPix, pixNoPerTrack=30;
   private int topAxisX2, topAxisY2; 
   private double axisDirectionY, axisDirectionZ;
   MappedSphere earth= new MappedSphere(40);
   public  static double sunAngularSpeed, earthAngularSpeed;
   private final static int obsDistanceToEarth=700; 
   private static int sunR=0; 
   private static int sunY_Observed, sunZ_Observed, sunX=-1, earthX=0;
   private static Pixel lum=new Pixel(-100,0,0);
   private static final int celestialRadius=200; 
   private static final double screenDistance = 500;
   private static final int staticSunRadius=23;
   private static int canvasCenterX,canvasCenterY;
   public static boolean newMonthDetector=false;
   public static boolean freezingDetector=false;
   public static boolean newLatitudeDetector=false;
   public static int horizonRadius =142;
   private Image mImage;
   
   private static TrackGenerator[] frontTracks=new TrackGenerator[7]; 
   private static TrackGenerator[] rearTracks=new TrackGenerator[7];    
 
   public static double obsAngleToEarth;//In radian
   public static double latitude;
   public static double longitude;
   public static double angleStep=0.05d;   
   public static double trackTime=0.0d;
   public static boolean newTrackTimeDetector= false;
   private static Pixel planeDirection = new Pixel(0, 0, 1);
    
   private static Thread SunMotion = null;
 
   /**Constructor.*/
   public SunEarthMovie(int sizeX, int sizeY)
   {
       canvasCenterX=sizeX/2;
       canvasCenterY=sizeY/2;
       setBackground(Color.black); 
       setSize(sizeX, sizeY);
   }
  
   /**Starts the simulation.*/
   public void start() {
                      SunMotion = new Thread(this, "SunEarthMovie");
                      SunMotion.start();
                       }

   /**Calculates the positions of the sun w.r.t an observer at a particular latitude at a particular fixed day in a year.*/
   public void run()	
   {  
       double x, y, z, startingTime;
       double angle=0.00d;
       int i;

       obsAngleToEarth=0.0;
       longitude=StandardConstLib.longitude;
       latitude=StandardConstLib.latitude*3.141592654/180;

       horizonRadius = (int) (celestialRadius*(double)screenDistance/obsDistanceToEarth);

       startingTime=-3*StandardConstLib.getQuarterYear()
                    +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3;

       axisDirectionY=-Math.cos(latitude);
       axisDirectionZ=Math.sin(latitude);
       topAxisX2 = (int)(horizonRadius*axisDirectionY);   
       topAxisY2 = (int)(horizonRadius*axisDirectionZ);

       trackTime = startingTime;
       sunAngularSpeed= StandardConstLib.sunAngularSpeed;
       noPix = earth.getPixelCount();

       earthAngularSpeed=StandardConstLib.earthAngularSpeed;
       earth.createMap("world1.jpg"); 

       for( i=0;i<7;i++)
       {
           frontTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude,
                                              obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);

           frontTracks[i].frontTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);

           rearTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude,
                                             obsDistanceToEarth, sunAngularSpeed,earthAngularSpeed);

           rearTracks[i].rearTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);
       }

       earth.mappingYZrotation(1.570796327-latitude,longitude);

       TrackTransformationMachine Tran = new TrackTransformationMachine(obsAngleToEarth,1.570796327-latitude,
                                             obsDistanceToEarth,sunAngularSpeed, earthAngularSpeed);
      
       while(true)
       {
           Tran.staticTrackYZ(angle, trackTime);
           x = Tran.getSunX();             	   
           y = Tran.getSunY();
           z = Tran.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
           sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

           lum=new Pixel((int)x,(int)y,(int)z);
           earth.lumination(lum);

           sunX=(int)x;
           sunY_Observed = canvasCenterX -sunR/2 + (int)y;
           sunZ_Observed = canvasCenterY -sunR/2 - (int)z;
           render(); 

           angle+=angleStep;
            
           if (newTrackTimeDetector)
           {
               Tran.staticTrackYZ(angle, trackTime);
               StaticTrack3.staticDayPanel.newDisplay();
               newTrackTimeDetector=false;
           }

           if (newLatitudeDetector)
           {
               for( i=0;i<7;i++)
               {
                   frontTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude,
                                                      obsDistanceToEarth,sunAngularSpeed, earthAngularSpeed);

                   frontTracks[i].frontTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);

                   rearTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude, obsDistanceToEarth,sunAngularSpeed, earthAngularSpeed);

                   rearTracks[i].rearTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);
               }  
     
               earth.mappingYZrotation(1.570796327-latitude,longitude);
               axisDirectionY=-Math.cos(latitude);
               axisDirectionZ=Math.sin(latitude);

               topAxisX2 = (int)(horizonRadius*axisDirectionY);   
               topAxisY2 = (int)(horizonRadius*axisDirectionZ);

               Tran = new TrackTransformationMachine(obsAngleToEarth,1.570796327-latitude,obsDistanceToEarth,
                                                     sunAngularSpeed,earthAngularSpeed);

               StaticTrack3.staticDayPanel.newDisplay();
               newLatitudeDetector=false;
           }
  
           while(freezingDetector)
           {
               Tran.staticTrackYZ(angle, trackTime);
               x = Tran.getSunX();             	   
               y = Tran.getSunY();
               z = Tran.getSunZ();

               x= celestialRadius*x;
               y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
               z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
               sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

               lum=new Pixel((int)x,(int)y,(int)z);
               earth.lumination(lum);
                   
               sunX=(int)x;
               sunY_Observed = canvasCenterX -sunR/2 + (int)y;
               sunZ_Observed = canvasCenterY -sunR/2 - (int)z;

               render();

               if (newTrackTimeDetector)
               {
                    Tran.staticTrackYZ(angle, trackTime);
                    StaticTrack3.staticDayPanel.newDisplay();
                    newTrackTimeDetector=false;
               }

               if (newLatitudeDetector)
               {
                   for( i=0;i<7;i++)
                   {
                       frontTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude,
                                                          obsDistanceToEarth,sunAngularSpeed, earthAngularSpeed);

                       frontTracks[i].frontTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);

                       rearTracks[i]= new TrackGenerator(pixNoPerTrack, obsAngleToEarth,1.570796327-latitude,
                                                         obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);

                       rearTracks[i].rearTrack(-3*StandardConstLib.getQuarterYear()+i*StandardConstLib.getQuarterYear()/3);
                   }

                   earth.mappingYZrotation(1.570796327-latitude,longitude);

                   axisDirectionY=-Math.cos(latitude);
                   axisDirectionZ=Math.sin(latitude);
                   topAxisX2 = (int)(horizonRadius*axisDirectionY);   
                   topAxisY2 = (int)(horizonRadius*axisDirectionZ);

                   Tran = new TrackTransformationMachine(obsAngleToEarth,1.570796327-latitude,obsDistanceToEarth,
                                                          sunAngularSpeed,earthAngularSpeed);
                   StaticTrack3.staticDayPanel.newDisplay();
                   newLatitudeDetector=false;
               }
           }
       }       
   }
   

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
          Dimension d= getSize();
          if (checkImage(d))
          {
              Graphics imageGraphics = mImage.getGraphics();
              imageGraphics.setColor(getBackground());
              imageGraphics.fillRect(0,0,d.width,d.height);
              paint(imageGraphics);
              g.drawImage(mImage,0,0,null);
              imageGraphics.dispose();
          }
          g.dispose();
       }
   }  

   /**Paints the sun-earth system to an observer at a particular latitude.*/
   public void paint(Graphics g)
   {
       int i,j,trackBrightness,planeRadius=6;
       float brightness, planeBrightness;
       Color color, planeColor;           
       Color c, c1;

       planeBrightness=Pixel.unitDotProduct(lum, planeDirection);

       if(planeBrightness<=0){planeColor=new Color(110,60,30);}
       else  {planeColor=new Color(255,220,180);}

       for(i=0;i<7;i++)
       { 
           for(j=0;j<pixNoPerTrack-1; j++)
           {
               trackBrightness=(int)((1+rearTracks[i].trPixel[j].getX()/(float)celestialRadius)*80)+70;

               c1 =new Color(trackBrightness, trackBrightness, trackBrightness);	
               g.setColor(c1);

               g.drawLine(canvasCenterX+ rearTracks[i].trPixel[j].getY(),canvasCenterY- rearTracks[i].trPixel[j].getZ(),
                          canvasCenterX+ rearTracks[i].trPixel[j+1].getY(),canvasCenterY- rearTracks[i].trPixel[j+1].getZ());
           }
       }

       if(sunX <0)
       {
           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed, sunZ_Observed, sunR, sunR);

           g.setColor(new Color(180,180,255));
           g.drawLine(canvasCenterX+topAxisX2, canvasCenterY-topAxisY2,canvasCenterX-topAxisX2, canvasCenterY+topAxisY2);

           for(i=0; i<noPix;i++)
           {   
               brightness=earth.spPixel[i].getPixelBrightness();
               color=earth.spPixel[i].getPixelColor();
               c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	

               g.setColor(c);
               g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(), canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
           }

           g.setColor(planeColor);
           g.fillOval(canvasCenterX-planeRadius,canvasCenterY-40,2*planeRadius, planeRadius);

           g.setColor(new Color(250,250,150));
           g.drawLine(canvasCenterX-horizonRadius,canvasCenterY,canvasCenterX+horizonRadius,canvasCenterY);

           for(i=0;i<7;i++)
           { 
               for(j=0;j<pixNoPerTrack-1; j++)
               {
                   trackBrightness=(int)((1+frontTracks[i].trPixel[j].getX()/(float)celestialRadius)*127);
                   c1 =new Color(trackBrightness, trackBrightness, trackBrightness);	
                   g.setColor(c1);

                   g.drawLine(canvasCenterX+ frontTracks[i].trPixel[j].getY(),canvasCenterY- frontTracks[i].trPixel[j].getZ(),
                              canvasCenterX+ frontTracks[i].trPixel[j+1].getY(),canvasCenterY- frontTracks[i].trPixel[j+1].getZ());
               }
           } 
       }
       else
       { 
           g.setColor(new Color(180,180,255));
	   g.drawLine(canvasCenterX+topAxisX2, canvasCenterY-topAxisY2,canvasCenterX-topAxisX2, canvasCenterY+topAxisY2);

           for(i=0; i<noPix;i++)
           {   
               brightness=earth.spPixel[i].getPixelBrightness();
               color=earth.spPixel[i].getPixelColor();
               c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
               g.setColor(c);
               g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
           }

           g.setColor(planeColor);
           g.fillOval(canvasCenterX-planeRadius,canvasCenterY-40,2*planeRadius, planeRadius);

 	   g.setColor(new Color(250,250,150));
           g.drawLine(canvasCenterX-horizonRadius,canvasCenterY,canvasCenterX+horizonRadius,canvasCenterY);

           for(i=0;i<7;i++)
           { 
               for(j=0;j<pixNoPerTrack-1; j++)
               {
                   trackBrightness=(int)((1+frontTracks[i].trPixel[j].getX()/(float)celestialRadius)*127);
                   c1 =new Color(trackBrightness, trackBrightness, trackBrightness);		

                   g.setColor(c1);
                   g.drawLine(canvasCenterX+ frontTracks[i].trPixel[j].getY(),canvasCenterY- frontTracks[i].trPixel[j].getZ(),
                              canvasCenterX+ frontTracks[i].trPixel[j+1].getY(),canvasCenterY- frontTracks[i].trPixel[j+1].getZ());
               }
           }

           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed,sunZ_Observed, sunR, sunR);
       }
   }

   protected boolean checkImage(Dimension d)
   {
       if (d.width==0 || d.height == 0) return false;
       if (mImage == null || mImage.getWidth(null) != d.width || mImage.getHeight(null)!=d.height)
       {
           mImage=createImage(d.width,d.height);   
       }
       return true;
   }
}

/**This panel displays a world map and allows users to change the position of the observer by clicking on the world map.*/
class MapDetectorPanel extends Panel implements MouseListener
{
   BufferedImage mImage;
   Image mImageBackground;
   int imageWidth, imageHeight;
   int pointX=0, pointY=0;
   double longitudeRatio;    

    /**Constructor that creates a map in memory by opening a picture file.*/
   public MapDetectorPanel(String filename)
   { 
       try{
           InputStream in =getClass().getResourceAsStream(filename);
           JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
           mImage= decoder.decodeAsBufferedImage();
           in.close(); 
       }catch(Exception e){}  

       setSize(mImage.getWidth(), mImage.getHeight()); 
       addMouseListener(this); 
       imageWidth = mImage.getWidth();
       imageHeight = mImage.getHeight();
       pointX = (int)(imageWidth*(StandardConstLib.longitude+0.2)); 
       pointY = (int)(imageHeight*(0.5-(double)StandardConstLib.latitude/180)); 
   }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImageBackground=createImage(imageWidth, imageHeight);   
           Graphics imageGraphics = mImageBackground.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImageBackground,0,0,null);
           imageGraphics.dispose();
           g.dispose();
       }
   }

    /**Paints the map.*/
   public void paint(Graphics g)
   {
       g.drawImage(mImage, 0, 0, this);
       g.setColor(new Color(255,220,180));
       g.fillOval(pointX-5,pointY-3, 10, 5);
   }

   /**Sets the new position of the observer if a user selects a new position on the map*/
   public void mousePressed(MouseEvent e)
   {
       SunEarthMovie.latitude = 3.141592654*(0.5-e.getY()/(double)imageHeight);
       longitudeRatio= e.getX()/(double)imageWidth;

       if(longitudeRatio<=0.2)           
       {
           SunEarthMovie.longitude = longitudeRatio+0.8d;
       }          
       else
       {
           SunEarthMovie.longitude = longitudeRatio-0.2d;
       } 	

       SunEarthMovie.newLatitudeDetector=true; 
       pointX = e.getX(); 
       pointY = e.getY();          
       render();
   }

   public void mouseClicked(MouseEvent e) { }
   public void mouseEntered(MouseEvent e) { }
   public void mouseExited(MouseEvent e) { }
   public void mouseReleased(MouseEvent e) { }
}

class ClickOnMapPanel extends Panel
{
   public ClickOnMapPanel()
   {
       setSize(380,20);
   }

   public void paint(Graphics g)
   {
       g.setColor(Color.green);
       g.drawString("Click on the map to select the position of the observer.",1,10);
   }
}

/**This panel tells users the day of a year that the motion on the screen correspond to.*/
class StaticDayPanel extends Panel 
{        
   private static int dayOfTheYear, newDay=81;
   public static double cosRotation=0.00d, sinRotation=0.00d, dayLength, startingTime;
   private static String[] numberString={"0","1","2","3","4","5","6","7","8","9"};
   public static boolean newLatDetector=false;
 
   /**Constructor.*/
   public StaticDayPanel()
   {
       setSize(400,20);

       startingTime=-3*StandardConstLib.getQuarterYear()
                    +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3;

       newDay=81;
       sinRotation = Math.sin(1.570796327-StandardConstLib.latitude*3.141592654/180);
       cosRotation = Math.cos(1.570796327-StandardConstLib.latitude*3.141592654/180);
       dayLength = dayTime(startingTime);
   }
   
   /**Sets the new day, calculates the length of that day and calls paint() to display it.*/
   public void newDisplay()
   {      
       sinRotation = Math.sin(1.570796327-SunEarthMovie.latitude);
       cosRotation = Math.cos(1.570796327-SunEarthMovie.latitude);
       dayLength = dayTime(SunEarthMovie.trackTime);

       newDay = (int)(SunEarthMovie.trackTime*StandardConstLib.sunAngularSpeed*58.09155423 + 264.33333d);                    
       repaint();
   }	

   /**Displays the day of the year and the length of that day.*/
   public void paint(Graphics g)
   {             
       String[] lengthString =  new String[3];
       String finalString;

       dayLength = (float)((int)(dayLength*10))/10;
       lengthString[0]="Length of the day = ";
       lengthString[1]=Float.toString((float)dayLength);
       lengthString[2]=" hours";

       finalString = lengthString[0].concat(lengthString[1]);
       finalString = finalString.concat(lengthString[2]);
       g.setColor(Color.cyan);
       g.drawString(finalString,1,10);
   }

   /**Returns the length of the day give a time in the year and the latitude of the observer.*/
   public double dayTime(double time)
   { 
       double sinSun, cosSun;
       double maxHeightAboveHorizon, minHeightAboveHorizon;
       double dayTimeResult=0;
       double staticPosition, amplitude;

       sinSun=Math.sin(StandardConstLib.sunAngularSpeed*time);
       cosSun=Math.cos(StandardConstLib.sunAngularSpeed*time);

       amplitude = sinRotation*Math.sqrt(0.84099918+0.15900082*cosSun*cosSun);
       staticPosition = -cosRotation*0.398749068*sinSun;

       maxHeightAboveHorizon = staticPosition +amplitude;
       minHeightAboveHorizon = staticPosition -amplitude;

       if (maxHeightAboveHorizon <=0){dayTimeResult = 0.00d;}

       if (minHeightAboveHorizon >=0){dayTimeResult = 24.00d;}
	    
       if ((maxHeightAboveHorizon > 0) && (minHeightAboveHorizon < 0))
       {
           dayTimeResult = 24.05*maxHeightAboveHorizon/(maxHeightAboveHorizon-minHeightAboveHorizon);
       } 
       return dayTimeResult;
   }
}

/**This panel displays the user interface that allows users to select any particular day in a year.*/ 
class MonthPanel extends Panel implements MouseListener, MouseMotionListener
{
    private BufferedImage background;
    private BufferedImage[] button1 = new BufferedImage[12];
    private BufferedImage[] button2 = new BufferedImage[12];
    private Image mImageBackground;
    public int mouseX1, mouseY1;
    private int buttonX=66, buttonY=43, buttonWidth=53, buttonHeight=17, buttonXDis=61, buttonYDis=22;
    public int buttonChoice = 0, buttonHighlight=0;
    public boolean highlight=false, lock=false;
    private String[] file1 = {"Jan2.jpg","Feb2.jpg","Mar2.jpg","Apr2.jpg","May2.jpg","Jun2.jpg","Jul2.jpg","Aug2.jpg","Sep2.jpg","Oct2.jpg","Nov2.jpg","Dec2.jpg"};
    private String[] file2 = {"Jan3.jpg","Feb3.jpg","Mar3.jpg","Apr3.jpg","May3.jpg","Jun3.jpg","Jul3.jpg","Aug3.jpg","Sep3.jpg","Oct3.jpg","Nov3.jpg","Dec3.jpg"};
    private int panelHeight, panelWidth;
    private double quarterYear;

    /**Constructor.*/
    public MonthPanel(int sizeX, int sizeY)
    {
        int i;

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file1[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button1[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file2[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button2[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        try{
               InputStream in =getClass().getResourceAsStream("DatesMenu.jpg");
               JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
               background = decoder.decodeAsBufferedImage();
               in.close(); 
           }catch(Exception e){} 

        panelWidth = sizeX;
        panelHeight = sizeY;
        addMouseListener(this);
        addMouseMotionListener(this);

        setValue(StandardConstLib.startingMonth-1);
        SunEarthMovie.trackTime = -3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3;
        SunEarthMovie.newTrackTimeDetector=true;
    }

    /**Sets the month of the year and repaints the user interface.*/
    public void setValue(int value)
    {
        buttonChoice = value;
        render();
    }

    /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
    protected void render()
    {
        Graphics g=getGraphics();
        if(g!=null)
        {        
            mImageBackground=createImage(panelWidth, panelHeight);
            Graphics imageGraphics = mImageBackground.getGraphics();
            paint(imageGraphics);
            g.drawImage(mImageBackground,0,0,null);
            imageGraphics.dispose();
            g.dispose();
        }
    }

    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        int i;
        g.drawImage(background,0,0,this);
        g.drawImage(button1[buttonChoice],buttonX+buttonXDis*(buttonChoice%4), buttonY+(buttonChoice/4)*buttonYDis,this);

        if(highlight)
        {
            g.drawImage(button2[buttonHighlight],buttonX+buttonXDis*(buttonHighlight%4), buttonY+(buttonHighlight/4)*buttonYDis,this);
        }
        lock=false;
    }

    /**Resets the month of the year if a particular button corresponding to that month is pressed.*/
    public void mousePressed(MouseEvent e)
    {
        double tempY, tempX;
        int temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if((mouseX1>buttonX)&&(mouseX1<buttonX+buttonXDis*4)&&(mouseY1>buttonY)&&(mouseY1<buttonY+buttonYDis*3)&&!lock)
        {
            tempX = (double)(mouseX1-buttonX)/buttonXDis;
            tempY = (double)(mouseY1-buttonY)/buttonYDis;
            temp = (int)tempY*4+(int)tempX;

            if((tempX-(int)tempX<(double)buttonWidth/buttonXDis)&&(tempY-(int)tempY<(double)buttonHeight/buttonYDis)&&buttonChoice!=temp)
            {
                buttonChoice = temp;

                SunEarthMovie.trackTime = -3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3;
                SunEarthMovie.newTrackTimeDetector=true;
                lock=true;
                highlight=false;
                render();
            }
        }
    }

    public void mouseReleased(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){highlight=false; lock=false;render();}

    public void mouseDragged(MouseEvent e){}

    public void mouseMoved(MouseEvent e)
    {
        double tempY, tempX;
        int temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if((mouseX1>buttonX)&&(mouseX1<buttonX+buttonXDis*4)&&(mouseY1>buttonY)&&(mouseY1<buttonY+buttonYDis*3)&&!lock)
        {
            tempX = (double)(mouseX1-buttonX)/buttonXDis;
            tempY = (double)(mouseY1-buttonY)/buttonYDis;
            temp = (int)tempY*4+(int)tempX;

            if((tempX-(int)tempX<(double)buttonWidth/buttonXDis)&&(tempY-(int)tempY<(double)buttonHeight/buttonYDis)&&buttonChoice!=temp)
            {
                buttonHighlight = temp;
                highlight=true;
                lock=true;
                render();
            }
            else
            {
                highlight=false;
                lock=true;
                render();
            }
        }        
    }
}


/**This class contains user interface that allows users to freeze, unfreeze, slow down and speed up the applet.*/
class SunEarthControlPanel extends Canvas implements MouseListener
{
    private BufferedImage background;
    private BufferedImage[] speedImage = new BufferedImage[4];
    private BufferedImage[] timeImage = new BufferedImage[4];
    private Image mImage;
    private String[] speedPicName = {"speedL.jpg","speedN.jpg","speedMinusB.jpg","speedPlusB.jpg"};
    private String[] timePicName = {"freezetime1.jpg","freezetime2.jpg","unfreezetime1.jpg","unfreezetime2.jpg"};
    private int timeX = 0, timeY = 12;
    private int speedX = 145, speedY = 12;   
    private int speedW = 112, speedH =37;
    private int timeW = 121, timeH =37;
    private int height, width;

    public static int speedNo = 1, timeNo = 0, resetNo = 0;

    /**Constructor.*/
    public SunEarthControlPanel(int sizeX, int sizeY, int timePos, int speedPos)
    {
        timeX=timePos;
        speedX=speedPos;

        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(speedPicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   speedImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }
   
        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(timePicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   timeImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }

        width=sizeX;
        height=sizeY;
        setBackground(new Color(6,76,73));
        addMouseListener(this);
        setVisible(true);
    }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImage=createImage(width, height);   
           Graphics imageGraphics = mImage.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImage,0,0,this);
           imageGraphics.dispose();
           g.dispose();
       }
   }
     
    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        g.setColor(new Color(75,75,75));
        g.fillRect(0,0,750,5);
        g.setColor(new Color(0,109,104));
        g.fillRect(0,5,750,50);
        g.setColor(new Color(4,76,73));
        g.fillRect(0,55,750,64);
        g.drawImage(speedImage[speedNo], speedX, speedY, this);
        g.drawImage(timeImage[timeNo], timeX, timeY, this);
    }

    /**Detects and records the button that is pressed.*/
    public void mousePressed(MouseEvent e)
    {
       int mouseX, mouseY;
       mouseX = e.getX();
       mouseY = e.getY();

       if((mouseX>timeX)&&(mouseX<timeX+timeW)&&(mouseY>timeY)&&(mouseY<timeY+timeH))
       {
           timeNo+=1;
       }
       
       if(timeNo==0)
       {
           if((mouseX>speedX+3)&&(mouseX<speedX+56)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=3;
           }
       
           if((mouseX>speedX+61)&&(mouseX<speedX+111)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=2;
           }
       }
       render();
    }

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
 
    /**Resets the state of the applet when when a button that was previous pressed on is now released.*/
    public void mouseReleased(MouseEvent e)
    {
        if(timeNo==1)
        {
            timeNo=2; speedNo=0;
            SunEarthMovie.freezingDetector=true;
        }
        else if(timeNo==3)
        {
            timeNo=0; speedNo=1;
            SunEarthMovie.freezingDetector=false;
        }

        if(speedNo==2)
        {
            speedNo=1;
            if(SunEarthMovie.angleStep>0.251)
            {SunEarthMovie.angleStep-=0.125;}
            else
            { SunEarthMovie.angleStep=SunEarthMovie.angleStep/2;}
        }

        if(speedNo==3)
        {   
            speedNo=1;         
            if(SunEarthMovie.angleStep<3&&SunEarthMovie.angleStep>=0.125)
            { 
                SunEarthMovie.angleStep+=0.125;
            }
            else if(SunEarthMovie.angleStep<0.1251)
            {SunEarthMovie.angleStep*=2;}
        }

        render();
    }
}

/**This class stores important constants that are used in this applet.*/
class StandardConstLib
{    
   public final static int startingMonth=3;
   public final static double obsAngleToEarth=0.00d;
   public final static int latitude= 0;
   public final static double longitude = 0.5;
   public final static double sunAngularSpeed=0.00008291, earthAngularSpeed=0.03;

   public static double getQuarterYear()
   {
       return 1.570796327/sunAngularSpeed;
   }
}