/**This class generates a set of tracks that the sun orbits in at different months of the year.*/
public class TrackGenerator
{
   public Pixel[] trPixel=new Pixel[80];
   private TrackTransformationMachine trans;
   private static int obsDistanceToEarth;
   private static int celestialRadius=200;
   private static int screenDistance=500;
   public int noOfPixel;

   /**Constructor.*/
   public TrackGenerator(int noPix,double obs_Angle, int obs_Distance, double sun_AngSpeed, double earth_AngSpeed)
   {
       trans = new TrackTransformationMachine(obs_Angle, obs_Distance, sun_AngSpeed, earth_AngSpeed);
       noOfPixel=noPix;
       obsDistanceToEarth=obs_Distance;
   }

   /**Constructor that allows the tracks to be rotated about the X-axis.*/
   public TrackGenerator(int noPix, double obs_Angle, double YZrotationAngle,int obs_Distance, double sun_AngSpeed, double earth_AngSpeed)
   {
       trans = new TrackTransformationMachine(obs_Angle, YZrotationAngle, obs_Distance, sun_AngSpeed, earth_AngSpeed);
       noOfPixel=noPix;
       obsDistanceToEarth=obs_Distance;
   }

   /**Generates an array of Pixel that represents the orbit of the sun behind the earth as view by the users.*/
   public void rearTrack(double trackTime)					  
   {  
       int i;
       double x, y, z, angleStep;
       angleStep = 3.141592654/(noOfPixel-1);

       for(i=0;i<noOfPixel;i++)
       {
           trans.staticTrackYZ(angleStep*i+3.141592654, trackTime); 

           x = trans.getSunX();             	   
           y = trans.getSunY();
           z = trans.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;

           trPixel[i]=new Pixel((int)x,(int)y,(int)z);
       }
   }

   /**Generates an array of Pixel that represents the orbit of the sun in front of the earth as view by the users.*/
   public void frontTrack(double trackTime)					  
   {
       int i;
       double x, y, z, angleStep;    
       angleStep = 3.141592654/(noOfPixel-1);  

       for(i=0;i<noOfPixel;i++)
       {
           trans.staticTrackYZ(angleStep*i, trackTime); 

           x = trans.getSunX();             	   
           y = trans.getSunY();
           z = trans.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;

           trPixel[i]=new Pixel((int)x,(int)y,(int)z);
       }
   }
}