/**This class defines a machine that calculates the motion of the sun w.r.t to earth on a fixed day in the year.*/
public class TrackTransformationMachine
{  
   private static final double earthAXISCOS=0.91706;//cos(23.5)
   private static final double earthAXISSIN=0.398749; //sin(23.5)
   private static final int celestialRadius=200; 
   private static final double screenDistance = 500;
   
   private int obs_Dis; 
   public double obs_AngleSin, obs_AngleCos, YZrotationSin, YZrotationCos; 
   private double sunAngular, earthAngular;
   private double x,y,z,sunRadius;

   /**Constructor.*/  
   public TrackTransformationMachine(double obs_Angle, int obs_Distance, double sun_AngSpeed, double earth_AngSpeed)
   {  
       sunAngular=sun_AngSpeed;
       earthAngular=earth_AngSpeed;
       obs_Dis=obs_Distance;
       obs_AngleSin=Math.sin(obs_Angle);
       obs_AngleCos=Math.cos(obs_Angle);
   }

   /**Constructor with the latitude of the observer.*/
   public TrackTransformationMachine(double obs_Angle, double YZrotAngle, int obs_Distance, double sun_AngSpeed, double earth_AngSpeed)
   {  
       sunAngular=sun_AngSpeed;
       earthAngular=earth_AngSpeed;
       obs_Dis=obs_Distance;
       obs_AngleSin=Math.sin(obs_Angle);
       obs_AngleCos=Math.cos(obs_Angle);
       YZrotationSin=Math.sin(YZrotAngle);
       YZrotationCos=Math.cos(YZrotAngle);
   }

   /**Calculates the position of the sun to the earth at a given time on a fixed day.*/
   public void staticTrack(double angle, double trackTime)
   {
       double sinSun, cosSun, sinEarth, cosEarth, smallRadius;

       sinSun = Math.sin(sunAngular*trackTime);
       cosSun = Math.cos(sunAngular*trackTime);
       sinEarth = Math.sin(angle);
       cosEarth = Math.cos(angle);

       smallRadius = Math.sqrt(1-earthAXISSIN*sinSun*earthAXISSIN*sinSun);

       x= obs_AngleCos*(smallRadius*sinEarth)-earthAXISSIN*sinSun*obs_AngleSin;
       y= smallRadius*cosEarth;
       z= -obs_AngleSin*(smallRadius*sinEarth)-earthAXISSIN*sinSun*obs_AngleCos;
   }

   /**Calculates the position of the sun w.r.t an observer on the north pole at a particular time on a given day, follows by a rotation of the sun about the earth about the X-axis.*/
   public void staticTrackYZ(double angle, double trackTime)
   {
       double sinSun, cosSun, sinEarth, cosEarth, smallRadius;

       sinSun = Math.sin(sunAngular*trackTime);
       cosSun = Math.cos(sunAngular*trackTime);
       sinEarth = Math.sin(angle);
       cosEarth = Math.cos(angle);

       smallRadius = Math.sqrt(1-earthAXISSIN*sinSun*earthAXISSIN*sinSun);

       x= smallRadius*sinEarth;
       y= smallRadius*cosEarth*YZrotationCos+YZrotationSin*earthAXISSIN*sinSun;
       z= YZrotationSin*smallRadius*cosEarth-earthAXISSIN*sinSun*YZrotationCos;
   }

   /**Calculates the position of the sun w.r.t an observer on the north pole at a particular time, follows by rotations of the sun first about the X-axis and then about the Y-axis.*/
   public void staticTrack3D(double angle, double trackTime)// YZrot followed by ZXrot	
   {
       double sinSun, cosSun, sinEarth, cosEarth, smallRadius;
       sinSun = Math.sin(sunAngular*trackTime);
       cosSun = Math.cos(sunAngular*trackTime);
       sinEarth = Math.sin(angle);
       cosEarth = Math.cos(angle);                   

       smallRadius = Math.sqrt(1-earthAXISSIN*sinSun*earthAXISSIN*sinSun);
            
       x = obs_AngleCos*(smallRadius*sinEarth) + obs_AngleSin*YZrotationSin*smallRadius*cosEarth
          -earthAXISSIN*sinSun*obs_AngleSin*YZrotationCos;

       y= smallRadius*cosEarth*YZrotationCos+YZrotationSin*earthAXISSIN*sinSun;

       z= -obs_AngleSin*(smallRadius*sinEarth) + YZrotationSin*obs_AngleCos*smallRadius*cosEarth 
          -earthAXISSIN*sinSun*obs_AngleCos*YZrotationCos;
   }

   /**Returns the x-position of the sun*/   
   public double getSunX()   {return x;}

   /**Returns the y-position of the sun*/   
   public double getSunY()   {return y;}

   /**Returns the Z-position of the sun*/   
   public double getSunZ()   {return z;}	
}

