import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;
import java.awt.font.TextAttribute;
import java.text.*;

/**This applet simulates the motion of sun and 12 zodiac signs w.r.t the earth in the course of a year.*/
public class Ecliptic extends Applet
{  
   public SunEarthMovie movieFrame;
   public static MonthPanel monthPanel;
   public DayPanel dayPanel;  
   public SunEarthControlPanel staticEarthControlPanel;  
   public PicturePanel title, signPanel;

   /**Starts this applet by initializing all panels and processes needed in this applet.*/
   public void init()
   { 
       setBackground(Color.black);
       setLayout(null);

       movieFrame = new SunEarthMovie(320,320);
       monthPanel = new MonthPanel(56,378);
       dayPanel = new DayPanel();
       staticEarthControlPanel = new SunEarthControlPanel(525,50,126,256);
       title= new PicturePanel("Title.jpg");
       signPanel= new PicturePanel("signPanel.jpg");

       add(monthPanel);
       add(movieFrame); 
       add(dayPanel);
       add(staticEarthControlPanel);
       add(title);
       add(signPanel);

       setVisible(true);
       monthPanel.setBounds(0,60,56,378);
       staticEarthControlPanel.setBounds(0,434,750,116);
       movieFrame.setBounds(80,70,320,320);  
       dayPanel.setBounds(88,408,370,22);
       signPanel.setBounds(411,60,339,378);
       title.setBounds(0,0,750,60);

       setVisible(true);     
       movieFrame.start();
       dayPanel.start();
   }
}

/**This window display the motion of the sun and 12 zodiac signs w.r.t the earth in the course of a year.*/
class SunEarthMovie extends Canvas implements Runnable
{  
   private static int noPix;
   private static int noOfPixInEcliptic;
   public MappedSphere earth= new MappedSphere(34);
   public EclipticGenerator ecliptic = new EclipticGenerator(81, 1300);
   public ZodiacMarker zodiacMarker = new ZodiacMarker(1300);
   public static Horoscope horo = new Horoscope(1300, 16);
   public static double sunAngularSpeed, earthAngularSpeed;
   private final static int obsDistanceToEarth=1300; 
   private static int sunR=0; 
   private static int sunY_Observed, sunZ_Observed, sunX=-1, earthX=0;
   private static int orbitRadius;
   private static Pixel lum=new Pixel(-100,0,0);
   private static final int celestialRadius=180; 
   private static final double screenDistance = 1000;
   private static final int staticSunRadius=24;
   private static int canvasCenterX,canvasCenterY;
   private static int furthestHoro;
   public static boolean freezingDetector=false;
   private Image mImage;
   public static double oldMonth, oneMonth;
  
   public static double absoluteTime;
   public static double obsAngleToEarth;//In radian
   public static double timeStep=3.01d;
   Color sourceColor= new Color(255,255,200);
          
   public static Thread SunMotion = null;
   
   /**Constructor.*/
   public SunEarthMovie(int sizeX, int sizeY)
   {
       canvasCenterX=sizeX/2;
       canvasCenterY=sizeY/2;
       setBackground(Color.black); 
       setSize(sizeX, sizeY);                       
   }

   /**Starts the simulation.*/
   public void start() {
                      SunMotion = new Thread(this, "SunEarthMovie");
                      SunMotion.start();
                       }

   /**Calculates the positions of the sun and 12 zodiac signs w.r.t the earth at a particular time in a year.*/
   public void run()
   {  
       double x, y, z, startingTime,thirteenMonth,angle,angleIn2Pi;
       int shift;

       obsAngleToEarth=0;

       startingTime=-3.230136986*StandardConstLib.getQuarterYear()
                    +StandardConstLib.startingMonth*StandardConstLib.getQuarterYear()/3;

       sunAngularSpeed= StandardConstLib.sunAngularSpeed;
       earthAngularSpeed=StandardConstLib.earthAngularSpeed;

       oneMonth = StandardConstLib.getQuarterYear()/3;            
       oldMonth = startingTime;
       thirteenMonth = 1.103196347*StandardConstLib.getQuarterYear();

       ecliptic.rotateEcliptic(0.0d);
       zodiacMarker.rotateZodiacMarker(0.00d);
       horo.rotateHoro(0.0d);

       earth.createMap("world1.jpg");   
       earth.mappingXZrotation(obsAngleToEarth,0.5);

       noPix = earth.getPixelCount();
       noOfPixInEcliptic = ecliptic.noOfPixel;

       TransformationMachine Tran = new TransformationMachine(obsAngleToEarth, 			obsDistanceToEarth, sunAngularSpeed, earthAngularSpeed);
       furthestHoro = 9; 

       orbitRadius=(int)Tran.getOrbitRadius();
           
       for(absoluteTime=startingTime;absoluteTime<=150000;absoluteTime+=timeStep)
       {            
           if((absoluteTime-oldMonth)>oneMonth)
           {
              if(absoluteTime>thirteenMonth)
              {
                 Ecliptic.monthPanel.setValue(0);
                 absoluteTime-=4*StandardConstLib.getQuarterYear();
              }
              else
              {
                  Ecliptic.monthPanel.setValue(Ecliptic.monthPanel.buttonChoice+1);
              }
              oldMonth=-3*StandardConstLib.getQuarterYear()
                       +(Ecliptic.monthPanel.buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                       -0.230136986*StandardConstLib.getQuarterYear();
           }

           Tran.transform(absoluteTime);
            	   
           x = Tran.getSunX();
           y = Tran.getSunY();
           z = Tran.getSunZ();

           x= celestialRadius*x;
           y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
           z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
           sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

           lum=new Pixel((int)x,(int)y,(int)z);
           earth.lumination(lum);
           sunX=(int)x;
           sunY_Observed = canvasCenterX -sunR/2 + (int)y;
           sunZ_Observed = canvasCenterY -sunR/2 - (int)z;

           angle = absoluteTime*earthAngularSpeed;
           ecliptic.rotateEcliptic(angle);
           zodiacMarker.rotateZodiacMarker(angle);
           horo.rotateHoro(angle);

           angleIn2Pi = angle/6.283185308-(long)(angle/6.283185308);
           if(angleIn2Pi<0){angleIn2Pi++;}
           shift = (int)(12*angleIn2Pi);

           furthestHoro = 9 + shift;
           if(furthestHoro>11){furthestHoro-=12;}
           render(); 
                
           while(freezingDetector)
           {
               Tran.transform(absoluteTime);

               x = Tran.getSunX();             	   
               y = Tran.getSunY();
               z = Tran.getSunZ();

               x= celestialRadius*x;
               y= (screenDistance/(obsDistanceToEarth-x))*y*celestialRadius;
               z= (screenDistance/(obsDistanceToEarth-x))*z*celestialRadius;
               sunR = (int)((screenDistance/(obsDistanceToEarth-x))*staticSunRadius); 

               lum=new Pixel((int)x,(int)y,(int)z);
               earth.lumination(lum);

               sunX=(int)x;
               sunY_Observed = canvasCenterX -sunR/2 + (int)y;
               sunZ_Observed = canvasCenterY -sunR/2 - (int)z;

               angle = absoluteTime*earthAngularSpeed;

               ecliptic.rotateEcliptic(angle);
               zodiacMarker.rotateZodiacMarker(angle);
               horo.rotateHoro(angle);

               angleIn2Pi = angle/6.283185308-(long)(angle/6.283185308);
               if(angleIn2Pi<0){angleIn2Pi++;}
               shift = (int)(12*angleIn2Pi);

               furthestHoro = 9 + shift;
               if(furthestHoro>11){furthestHoro-=12;}

               render();  	  
           }
       }                        
   }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           Dimension d= getSize();
           if (checkImage(d))
           {
               Graphics imageGraphics = mImage.getGraphics();
               imageGraphics.setColor(getBackground());
               imageGraphics.fillRect(0,0,d.width,d.height);
               paint(imageGraphics);
               g.drawImage(mImage,0,0,null);
               imageGraphics.dispose();
           }
           g.dispose();
       }
   }   

   /**Paints the sun-earth and zodiac signs motion.*/
   public void paint(Graphics g)
   {
       int i, j,k, zodiacSize=2;
       float brightness, eBrightness;
       Color color;           
       Color c;

       if(sunX <0)
       {
           for(i=((noOfPixInEcliptic-1)/2);i<noOfPixInEcliptic-1;i++)
           {                         
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()<0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }                   
         
           for(k=(furthestHoro-3); k < (furthestHoro + 4);k++)
           {
               j=k;	
	       if(j>11){j-=12;}
	       if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           } 

           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed, sunZ_Observed, sunR, sunR);

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()>0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }

           g.setColor(new Color(200,200,255));
           g.drawLine(canvasCenterX, canvasCenterY-orbitRadius,canvasCenterX, canvasCenterY+orbitRadius);

           g.setColor(new Color(120,120,150));
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius, 2*orbitRadius,2*orbitRadius);
              
           for(i=0; i<noPix;i++)
           {   
               brightness=earth.spPixel[i].getPixelBrightness();
               color=earth.spPixel[i].getPixelColor();
               c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	

               g.setColor(c);
               g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
           }

           for(i=0;i<((noOfPixInEcliptic-1)/2);i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(k=(furthestHoro-8); k < (furthestHoro - 3);k++)
           {	
               j=k;	
	       if(j>11){j-=12;}
	       if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           } 

           g.setColor(new Color(200,200,255));
           g.drawLine(canvasCenterX-orbitRadius, canvasCenterY,canvasCenterX+orbitRadius, canvasCenterY);
       }
       else
       { 
           for(i=((noOfPixInEcliptic-1)/2);i<noOfPixInEcliptic-1;i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness), (int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()<0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }

           for(k=(furthestHoro-3); k < (furthestHoro + 4);k++)
           {
               j=k;	
	       if(j>11){j-=12;}
	       if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
                   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           } 

           g.setColor(new Color(200,200,255));
           g.drawLine(canvasCenterX, canvasCenterY-orbitRadius,canvasCenterX, canvasCenterY+orbitRadius);

           g.setColor(new Color(120,120,150));
           g.drawOval(canvasCenterX-orbitRadius,canvasCenterY-orbitRadius, 2*orbitRadius,2*orbitRadius);

           for(i=0; i<noPix;i++)
           {   
               brightness=earth.spPixel[i].getPixelBrightness();
               color=earth.spPixel[i].getPixelColor();
               c=new Color((int)(brightness*color.getRed()),(int)(brightness*color.getGreen()),(int)(brightness*color.getBlue()));	
               g.setColor(c);
               g.fillRect(canvasCenterX-1+earth.spPixel[i].getY(),canvasCenterY-1-earth.spPixel[i].getZ(), 1,1);
           }

           for(i=0;i<((noOfPixInEcliptic-1)/2);i++)
           {
               eBrightness = ecliptic.originalEclipticPix[i].getPixelBrightness();
               color = new Color((int)(220*eBrightness),(int)(220*eBrightness),(int)(253*eBrightness));
               g.setColor(color);
               g.drawLine(canvasCenterX+ecliptic.trPixel[i].getY(),canvasCenterY-ecliptic.trPixel[i].getZ(),
                          canvasCenterX+ecliptic.trPixel[i+1].getY(),canvasCenterY-ecliptic.trPixel[i+1].getZ());
           }

           for(i=0;i<12;i++)
           {
               g.setColor(Color.white);
               if(zodiacMarker.marker[i].getX()>0)
               {
                   g.drawLine(canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()-3,
                              canvasCenterX+ zodiacMarker.marker[i].getY(),canvasCenterY- zodiacMarker.marker[i].getZ()+3);
               }
           }

           for(k=(furthestHoro-8); k < (furthestHoro - 3);k++)
           {	
               j=k;	
	       if(j>11){j-=12;}
	       if(j<0) {j+=12;}

               for(i=0; i<horo.noPixHoroSphere;i++)
               {   
            	   color=horo.horoSphere[j].spPixel[i].getPixelColor();
                   g.setColor(color);
                   g.fillRect(canvasCenterX+ horo.horoPosition[j].getY()+horo.horoSphere[j].spPixel[i].getY(),
                              canvasCenterY- horo.horoPosition[j].getZ()-horo.horoSphere[j].spPixel[i].getZ(), 1, 1);
               }
           }

           g.setColor(new Color(200,200,255));
           g.drawLine(canvasCenterX-orbitRadius, canvasCenterY,canvasCenterX+orbitRadius, canvasCenterY);

           g.setColor(Color.yellow);
           g.fillOval(sunY_Observed,sunZ_Observed, sunR, sunR);
       }
   }

   protected boolean checkImage(Dimension d)
   {
       if (d.width==0 || d.height == 0) return false;
       if (mImage == null || mImage.getWidth(null) != d.width || mImage.getHeight(null)!=d.height)
       {
           mImage=createImage(d.width,d.height);   
       }
       return true;
   }
}

/**This class contains user interface that allows users to freeze, unfreeze, slow down and speed up the applet.*/
class SunEarthControlPanel extends Canvas implements MouseListener
{
    private BufferedImage background;
    private BufferedImage[] speedImage = new BufferedImage[4];
    private BufferedImage[] timeImage = new BufferedImage[4];
    private Image mImage;
    private String[] speedPicName = {"speedL.jpg","speedN.jpg","speedMinusB.jpg","speedPlusB.jpg"};
    private String[] timePicName = {"freezetime1.jpg","freezetime2.jpg","unfreezetime1.jpg","unfreezetime2.jpg"};
    private int timeX = 0, timeY = 12;
    private int speedX = 145, speedY = 12;   
    private int speedW = 112, speedH =37;
    private int timeW = 121, timeH =37;
    private int height, width;

    public static int speedNo = 1, timeNo = 0, resetNo = 0;

    /**Constructor.*/
    public SunEarthControlPanel(int sizeX, int sizeY, int timePos, int speedPos)
    {
        timeX=timePos;
        speedX=speedPos;

        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(speedPicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   speedImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }
   
        for(int i=0;i<4;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(timePicName[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   timeImage[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){}
        }

        width=sizeX;
        height=sizeY;
        setBackground(new Color(6,76,73));
        addMouseListener(this);
        setVisible(true);
    }

   /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
   protected void render()
   {
       Graphics g=getGraphics();
       if(g!=null)
       {
           mImage=createImage(width, height);   
           Graphics imageGraphics = mImage.getGraphics();
           paint(imageGraphics);
           g.drawImage(mImage,0,0,this);
           imageGraphics.dispose();
           g.dispose();
       }
   }
     
    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        g.setColor(new Color(75,75,75));
        g.fillRect(0,0,750,5);
        g.setColor(new Color(0,109,104));
        g.fillRect(0,5,750,50);
        g.setColor(new Color(4,76,73));
        g.fillRect(0,55,750,64);
        g.drawImage(speedImage[speedNo], speedX, speedY, this);
        g.drawImage(timeImage[timeNo], timeX, timeY, this);
    }

    /**Detects and records the button that is pressed.*/
    public void mousePressed(MouseEvent e)
    {
       int mouseX, mouseY;
       mouseX = e.getX();
       mouseY = e.getY();

       if((mouseX>timeX)&&(mouseX<timeX+timeW)&&(mouseY>timeY)&&(mouseY<timeY+timeH))
       {
           timeNo+=1;
       }
       
       if(timeNo==0)
       {
           if((mouseX>speedX+3)&&(mouseX<speedX+56)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=3;
           }
       
           if((mouseX>speedX+61)&&(mouseX<speedX+111)&&(mouseY>speedY+17)&&(mouseY<speedY+35))
           {
               speedNo=2;
           }
       }
       render();
    }

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}
 
    /**Resets the state of the applet when when a button that was previous pressed on is now released.*/
    public void mouseReleased(MouseEvent e)
    {
        if(timeNo==1)
        {
            timeNo=2; speedNo=0;
            SunEarthMovie.freezingDetector=true;
        }
        else if(timeNo==3)
        {
            timeNo=0; speedNo=1;
            SunEarthMovie.freezingDetector=false;
        }

        if(speedNo==2)
        {
            speedNo=1;
            if(SunEarthMovie.timeStep>0.51)
            {SunEarthMovie.timeStep-=0.251;}
            else
            { SunEarthMovie.timeStep=SunEarthMovie.timeStep/2;}
        }

        if(speedNo==3)
        {   
            speedNo=1;         
            if(SunEarthMovie.timeStep<15&&SunEarthMovie.timeStep>=0.251)
            { 
                SunEarthMovie.timeStep+=0.251;
            }
            else if(SunEarthMovie.timeStep<0.251)
            {SunEarthMovie.timeStep*=2;}
        }

        render();
    }
}

/**This panel tells users the day of a year that the motion on the screen correspond to.*/
class DayPanel extends Panel implements Runnable
{
    private Thread dayTimer;       
    private int dayOfTheYear, newDay=61;
    private static String[] numberString={"0","1","2","3","4","5","6","7","8","9"};

    //comments:absoluteTime=0 at September Equinox =264.33days in one year.
    
    /**Constructor.*/
    public DayPanel()
    {
        setSize(100,20);
    }

    /**Starts the panel.*/
    public void start()
    {
        dayTimer = new Thread(this, "DayPanel");
        dayTimer.start();
    }  

    /**Calculates the time and displays it if it is a new day.*/
    public void run()
    {
        dayOfTheYear= ((StandardConstLib.startingMonth-1)*365)/12;
        newDay=dayOfTheYear;

        while(dayTimer.isAlive())
        {
            newDay = (int)(SunEarthMovie.absoluteTime*StandardConstLib.sunAngularSpeed*58.09155423+264.33334d+1);

            if(newDay!=dayOfTheYear)
            {
                 repaint();
                 dayOfTheYear=newDay;
            }

            try{
                   Thread.sleep(300);
               }catch(InterruptedException e){} 
        }
    }
	
    /**Paints the day on the screen.*/
    public void paint(Graphics g)
    {
        int newDayCounter;
        double angle;
        float floatAngle;
        String finalString;
        g.setColor(Color.cyan);

        newDayCounter=newDay;

        finalString=Integer.toString(newDayCounter);
        finalString=finalString.concat(" of 365 days");
        g.drawString(finalString, 5,10);
            
        angle = -57.2957795*Math.asin(0.398749068*Math.sin(StandardConstLib.sunAngularSpeed*SunEarthMovie.absoluteTime));

        finalString = "Declination angle of the Sun = ";
        floatAngle = (float)((int)(angle*10))/10;	    
        finalString = finalString.concat(Float.toString(floatAngle));

        g.drawString( finalString, 125,10);         
    }
}

/**This panel displays the user interface that allows users to select any particular month in a year.*/ 
class MonthPanel extends Panel implements MouseListener, MouseMotionListener
{
    private BufferedImage background;
    private BufferedImage[] button = new BufferedImage[12];
    private Image mImageBackground;
    public int mouseX1, mouseY1;
    private int buttonX=0, buttonY=332, buttonWidth=50, buttonHeight=20, buttonYDis=24;
    public int buttonChoice = 0;
    private String[] file = {"Mth01.jpg","Mth02.jpg","Mth03.jpg","Mth04.jpg","Mth05.jpg","Mth06.jpg","Mth07.jpg","Mth08.jpg","Mth09.jpg","Mth10.jpg","Mth11.jpg","Mth12.jpg"};
    private int panelHeight, panelWidth;
    private double quarterYear;

    /**Constructor.*/
    public MonthPanel(int sizeX, int sizeY)
    {
        int i;

        for(i=0;i<12;i++)
        {
            try{
                   InputStream in =getClass().getResourceAsStream(file[i]);
                   JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
                   button[i] = decoder.decodeAsBufferedImage();
                   in.close(); 
               }catch(Exception e){} 
        }

        try{
               InputStream in =getClass().getResourceAsStream("MonthButton.jpg");
               JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
               background = decoder.decodeAsBufferedImage();
               in.close(); 
           }catch(Exception e){} 

        panelWidth = sizeX;
        panelHeight = sizeY;
        addMouseListener(this);
        addMouseMotionListener(this);

        setValue(StandardConstLib.startingMonth-1);

        quarterYear=StandardConstLib.getQuarterYear();
           
        SunEarthMovie.absoluteTime =-3*quarterYear+quarterYear*StandardConstLib.startingMonth/3.00d
                                          -0.230136986*quarterYear;//0.230136986 = 21days
    }

    /**Sets the month of the year and repaints the user interface.*/
    public void setValue(int value)
    {
        buttonChoice = value;
        render();
    }

    /**Invokes the graphics memory and calls paint() to fill the memory with image.*/
    protected void render()
    {
        Graphics g=getGraphics();
        if(g!=null)
        {
            mImageBackground=createImage(panelWidth, panelHeight);
            Graphics imageGraphics = mImageBackground.getGraphics();
            paint(imageGraphics);
            g.drawImage(mImageBackground,0,0,null);
            imageGraphics.dispose();
            g.dispose();
        }
    }     

    /**Paints the user interface.*/
    public void paint(Graphics g)
    {
        int i;
        g.drawImage(background,0,0,this);
        g.drawImage(button[buttonChoice],buttonX, buttonY-(buttonChoice+1)*buttonYDis,this);
    }

    /**Resets the month of the year if a particular button corresponding to that month is pressed.*/
    public void mousePressed(MouseEvent e)
    {
        double temp;
        mouseY1 = e.getY();
        mouseX1 = e.getX();

        if(mouseX1>buttonX&&mouseX1<buttonX+buttonWidth)
        {
            temp = (double)(buttonY-mouseY1)/buttonYDis;
            if((temp-(int)temp<(double)buttonHeight/buttonYDis)&&buttonChoice!=(int)temp)
            {
                buttonChoice = (int)temp;

                SunEarthMovie.absoluteTime=-3*quarterYear+quarterYear*(double)(buttonChoice+1)/3.00d
                                           -0.230136986*quarterYear;
                           
                SunEarthMovie.oldMonth=-3*StandardConstLib.getQuarterYear()+(buttonChoice+1)*StandardConstLib.getQuarterYear()/3
                                       -0.230136986*StandardConstLib.getQuarterYear();
                render();
            }
        }
    }

    public void mouseReleased(MouseEvent e){}

    public void mouseClicked(MouseEvent e){}

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}

    public void mouseDragged(MouseEvent e){}

    public void mouseMoved(MouseEvent e){}
}

/**This class stores important constants that are used in this applet.*/
class StandardConstLib
{    
    public final static int startingMonth=3;
    public final static double obsAngleToEarth=0.00d;
    public final static double sunAngularSpeed=0.00008291, earthAngularSpeed=0.03034506;

    public static double getQuarterYear()
    {
        return 1.570796327/sunAngularSpeed;
    }
}