/**This class define an orbit that represents the ecliptic.*/
public class EclipticGenerator
{
   //the ecliptic is  left-hand-generated around z-axis.   
   private static final double earthAXISCOS=0.91706;//cos(23.5)
   private static final double earthAXISSIN=0.398749; //sin(23.5)
   public Pixel[] originalEclipticPix = new Pixel[111];
   public Pixel[] trPixel=new Pixel[111];
   private static int obsDistanceToEarth;
   private static int celestialRadius=180;
   private static int screenDistance=1000;
   public int noOfPixel;

    /**Constructor that creates an array of Pixel that represents the ecliptic.*/
   public EclipticGenerator(int noPix, int obs_Distance)
   {  
       int i;
       double angleStep, x, y, z;
       noOfPixel=noPix;
       float brightness;

       angleStep = 6.283185308/(noOfPixel-1); 
       obsDistanceToEarth=obs_Distance;

       for(i=0;i<noOfPixel;i++)
       {
           x = celestialRadius*Math.sin(angleStep*i);
           y = celestialRadius*earthAXISCOS*Math.cos(angleStep*i);
           z = celestialRadius*earthAXISSIN*Math.cos(angleStep*i);

           brightness = (float)(0.5+0.25*(1+x/celestialRadius));
           originalEclipticPix[i]=new Pixel((int)x,(int)y,(int)z);
           originalEclipticPix[i].setPixelBrightness(brightness);
       }
   }

   /**Calculates the position of individual point on the orbit that represents the ecliptic given an viewing angle.*/
   public void rotateEcliptic(double angle)					  
   {  
       int i, shift, remainder;
       double x, y, z, cosRotation, sinRotation, angleIn2Pi;

       angleIn2Pi = angle/6.283185308-(long)(angle/6.283185308);
       if(angleIn2Pi<0){angleIn2Pi++;}

       shift = (int)((noOfPixel-1)*angleIn2Pi);
       cosRotation = Math.cos(angle);
       sinRotation = Math.sin(angle);
 
       for(i=0;i<noOfPixel-shift;i++)
       {
           x = cosRotation*originalEclipticPix[i].getX()+sinRotation*originalEclipticPix[i].getY();
           y =-sinRotation*originalEclipticPix[i].getX()+cosRotation*originalEclipticPix[i].getY();	
           z = originalEclipticPix[i].getZ();

           y= (screenDistance/(obsDistanceToEarth-x))*y;
           z= (screenDistance/(obsDistanceToEarth-x))*z;

           trPixel[i+shift] = new Pixel((int)x, (int)y, (int)z);
       }

       for(i=noOfPixel-shift-1;i<noOfPixel-1;i++)
       {
           x = cosRotation*originalEclipticPix[i].getX()+sinRotation*originalEclipticPix[i].getY();  
           y =-sinRotation*originalEclipticPix[i].getX()+cosRotation*originalEclipticPix[i].getY();	
           z = originalEclipticPix[i].getZ();

           y= (screenDistance/(obsDistanceToEarth-x))*y;
           z= (screenDistance/(obsDistanceToEarth-x))*z;

           remainder = (i+shift)%(noOfPixel-1);
           trPixel[remainder] = new Pixel((int)x, (int)y, (int)z);
       }   
   }

}