/**This class defines a machine that calculates the position of the sun w.r.t to earth.*/
public class TransformationMachine
{  

   private static final double earthAXISCOS=0.91706;//cos(23.5)
   private static final double earthAXISSIN=0.398749; //sin(23.5)
   private static final int celestialRadius=180; 
   private static final double screenDistance = 1000;
   private int obs_Dis; 
   public double obs_AngleSin, obs_AngleCos; 
   private double sunAngular, earthAngular;
   private double x,y,z,sunRadius;

   /**Constructor*/
   public TransformationMachine(double obs_Angle, int obs_Distance, double sun_AngSpeed,
                                double earth_AngSpeed)
   {  
       sunAngular=sun_AngSpeed;
       earthAngular=earth_AngSpeed;
       obs_Dis=obs_Distance;
       obs_AngleSin=Math.sin(obs_Angle);
       obs_AngleCos=Math.cos(obs_Angle);
   }

   /**Calculates the relative position of the sun to the earth at a given time.*/
   public void transform(double time)	
   {
       double sinSun, cosSun, sinEarth, cosEarth;
       sinSun = Math.sin(sunAngular*time);
       cosSun = Math.cos(sunAngular*time);
       sinEarth = Math.sin(earthAngular*time);
       cosEarth = Math.cos(earthAngular*time);

       x= -obs_AngleCos*(cosEarth*cosSun + earthAXISCOS*sinSun*sinEarth)-earthAXISSIN*sinSun*obs_AngleSin;
       y= sinEarth*cosSun - cosEarth*earthAXISCOS*sinSun;
       z= obs_AngleSin*(cosEarth*cosSun + earthAXISCOS*sinSun*sinEarth)-earthAXISSIN*sinSun*obs_AngleCos;
   }

   /**Returns the x-position of the sun*/   
   public double getSunX()   {return x;}

   /**Returns the y-position of the sun*/   
   public double getSunY()   {return y;}

   /**Returns the z-position of the sun*/   
   public double getSunZ()   {return z;}

   /**Returns the apparent distance between the earth and the sun on the screen*/
   public double getOrbitRadius()
   {
      return (screenDistance/obs_Dis)*celestialRadius;
   }

   /**Returns the apparent height of orbit of the sun around the earth on the screen.*/
   public double getcelestXYelipHeight()
   {
       return Math.abs((screenDistance/obs_Dis)*celestialRadius*obs_AngleSin);
   }	
}

