import java.awt.*;

/**This class defines a pixel on the screen and it contains information of the direction of this pixel in 3D.
  *The direction of the pixel is direction in which the area represented by this pixel is facing. This direction is useful in determine the brightness of this area given a light source.
  */
public class Pixel 
{ 
   private int pixelX, pixelY, pixelZ, pixelWidth, pixelHeight;
   private float unitX, unitY, unitZ, pixelBrightness;
   private double vecRadius;
   private int originX, originY, originZ;
   private Color pixelColor;

   /**Constructor*/
   public Pixel(int X, int Y, int Z)
   {
       pixelX=X;
       pixelY=Y;
       pixelZ=Z;
       vecRadius = Math.sqrt(X*X+Y*Y+Z*Z);
       unitX=(float)(X/vecRadius);
       unitY=(float)(Y/vecRadius);
       unitZ=(float)(Z/vecRadius);
   }

   /**Constructor for a square pixel*/
   public Pixel(int X, int Y, int Z,int Size)
   {
       pixelX=X;
       pixelY=Y;
       pixelZ=Z;
       vecRadius = Math.sqrt(X*X+Y*Y+Z*Z);
       unitX=(float)(X/vecRadius);
       unitY=(float)(Y/vecRadius);
       unitZ=(float)(Z/vecRadius); 
       pixelWidth=Size;
       pixelHeight=Size;	   
   }

   /**Constructor with size of the pixel*/
   public Pixel(int X, int Y, int Z, int width, int height)
   {
       pixelX=X;
       pixelY=Y;
       pixelZ=Z;
       pixelWidth=width;
       pixelHeight=height;
       vecRadius = Math.sqrt(X*X+Y*Y+Z*Z);
       unitX=(float)(X/vecRadius);
       unitY=(float)(Y/vecRadius);
       unitZ=(float)(Z/vecRadius);
   }

   /**Constructor with size and color of the pixel*/
   public Pixel(int X, int Y, int Z,int Size, Color pixColor)
   {
       pixelX=X;
       pixelY=Y;
       pixelZ=Z;
       vecRadius = Math.sqrt(X*X+Y*Y+Z*Z);
       unitX=(float)(X/vecRadius);
       unitY=(float)(Y/vecRadius);
       unitZ=(float)(Z/vecRadius); 
       pixelWidth=Size;
       pixelHeight=Size; 
       pixelColor=pixColor;
   }

   /**Returns the direction of the reflected light beam when a beam from incidentB impinges on a pixel facing A.*/
   public static Pixel reflectedVector(Pixel A, Pixel incidentB)
   {
       int Vx, Vy, Vz;
       Pixel reflectedVector;
           
       Vx = (int)(-incidentB.pixelX + 2*(unitDotProduct(A,incidentB))*incidentB.vecRadius*A.unitX);

       Vy = (int)(-incidentB.pixelY + 2*(unitDotProduct(A,incidentB))*incidentB.vecRadius*A.unitY);

       Vz = (int)(-incidentB.pixelZ + 2*(unitDotProduct(A,incidentB))*incidentB.vecRadius*A.unitZ);
           
       reflectedVector = new Pixel(Vx, Vy, Vz); 

       return reflectedVector;
   }

   /**Returns the unit dot product between the direction of Pixel A and direction of Pixel B.*/
   public static float unitDotProduct(Pixel A, Pixel B)
   {
       return (A.unitX*B.unitX + A.unitY*B.unitY + A.unitZ*B.unitZ);                  
   }

   /**Sets the brightness of this pixel, 0 represents total darkness, 1 represents maximum brightness.*/
   public void setPixelBrightness(float x) {pixelBrightness = x;}

   /**Returns the brightness of this pixel.*/
   public float getPixelBrightness() {return pixelBrightness;}

   /**Sets the color of this pixel.*/
   public void setPixelColor(Color x) {pixelColor = x;}

   /**Returns the color of this pixel.*/
   public Color getPixelColor() {return pixelColor;}

   /**Returns the X-position of this pixel*/
   public int getX()   { return pixelX;}

   /**Returns the Y-position of this pixel*/
   public int getY()   { return pixelY;}

   /**Returns the Z-position of this pixel*/
   public int getZ()   { return pixelZ;}

   /**Returns the width of this pixel on the screen*/
   public int getPixelWidth() {return pixelWidth;}

   /**Returns the height of this pixel on the screen*/
   public int getPixelHeight() {return pixelHeight;}

   /**Returns the x-component of the unit vector of this pixel*/
   public float getUnitX()   {return unitX;}

   /**Returns the y-component of the unit vector of this pixel*/
   public float getUnitY()   {return unitY;}

   /**Returns the z-component of the unit vector of this pixel*/
   public float getUnitZ()   {return unitZ;}

   /**Returns the length of this vector pixel*/
   public double getVecRadius() {return vecRadius;}
}