/**This machine calculates the length and direction of the shadow of a vertical dipole given the position of the sun.*/
public class ShadowGenerator
{
   private Pixel pole;
   private Pixel rotatedPole;
   private Pixel shadow;
   private double cosViewAngle, sinViewAngle;
   private Pixel unitZ;
   private int planeRadius;

   /**Constructor.*/
   public ShadowGenerator(Pixel inputpole, double viewAngle, int planeRad)
   {
       cosViewAngle = Math.cos(viewAngle);
       sinViewAngle = Math.sin(viewAngle);
       pole = inputpole;
       unitZ = new Pixel(0,0,1);
       planeRadius = planeRad;
   }
   
   /**Rotates the sundial given a viewing angle.*/
   public void rotatePole()
   {
       int x, y, z;	   
       x = (int)(cosViewAngle*pole.getX()+sinViewAngle*pole.getZ());
       y = pole.getY();
       z = (int)(-sinViewAngle*pole.getX()+cosViewAngle*pole.getZ());

       rotatedPole = new Pixel(x, y, z);
   }

   /**Generates the shadow given the position of the sun.*/
   public void createShadow(Pixel lum)
   {
       int shadowX, shadowY, shadowZ;
       double n;

       n = (double)(pole.getZ()/Pixel.unitDotProduct(lum, unitZ));           
       shadowX =  (int)(pole.getX() - n*lum.getUnitX());
       shadowY =  (int)(pole.getY() - n*lum.getUnitY());

       shadow =  new Pixel(shadowX, shadowY, 0);

       if (shadow.getVecRadius()>planeRadius)
       {
           shadow = new Pixel((int)((planeRadius+2)*shadow.getUnitX()),(int)((planeRadius+2)*shadow.getUnitY()), 0);
       }
   }

   /**Rotates the shadow given the viewing angle of the observer.*/
   public void rotateShadow()
   {
       int x, y, z;
	   
        x = (int)(cosViewAngle*shadow.getX()+sinViewAngle*shadow.getZ());
        y = shadow.getY();
        z = (int)(-sinViewAngle*shadow.getX()+cosViewAngle*shadow.getZ());

        shadow = new Pixel(x, y, z);
   }
 
   /**Changes the direction of the sundial.*/
   public void changePole(Pixel inputPole) {pole = inputPole;}

   /**Returns the X-direction of the sundial.*/
   public int getRotatedPoleX() {return rotatedPole.getX();}

   /**Returns the Y-direction of the sundial.*/
   public int getRotatedPoleY() {return rotatedPole.getY();}

   /**Returns the Z-direction of the sundial.*/
   public int getRotatedPoleZ() {return rotatedPole.getZ();}

   /**Returns the X-direction of the shadow.*/
   public int getShadowX() {return shadow.getX();}  

   /**Returns the Y-direction of the shadow.*/
   public int getShadowY() {return shadow.getY();}

   /**Returns the Z-direction of the shadow.*/
   public int getShadowZ() {return shadow.getZ();}
}